@@ -1,6 +1,6 @@
 /*
 #   oggdec.c: ogg file parser for xlplayer
-#   Copyright (C) 2008-2009 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2008-2013 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -26,19 +26,14 @@
 #include "xlplayer.h"
 #include "oggdec.h"
 #include "ogg_vorbis_dec.h"
+#include "ogg_opus_dec.h"
 #include "ogg_flac_dec.h"
 #include "ogg_speex_dec.h"
+#include "vorbistagparse.h"
 
-#define TRUE 1
-#define FALSE 0
 #define ACCEPTED 1
 #define REJECTED 0
 
-#define readint(buf, base) (((buf[base+3]<<24)&0xff000000)| \
-                                    ((buf[base+2]<<16)&0xff0000)| \
-                                    ((buf[base+1]<<8)&0xff00)| \
-                                     (buf[base]&0xff))
-
 int oggdec_get_next_packet(struct oggdec_vars *self)
     {
     char *buffer;
@@ -416,184 +411,6 @@ static int flac_get_samplerate(struct oggdec_vars *self)
 
 #ifdef HAVE_SPEEX
 
-static int speex_comment_count(char *comments, int length, char *match_string)
-    {
-    char *c=comments;
-    int len, i, nb_fields, count = 0;
-    char *end;
-
-    if (length < 8)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return -1;
-        }
-    
-    end = c + length;
-    len = readint(c, 0);
-    c += 4;
-    
-    if (c + len > end)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return -1;
-        }
-
-    c += len;
-    if (c + 4 > end)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return -1;
-        }
-
-    nb_fields = readint(c, 0);
-    c += 4;
-    for (i = 0; i < nb_fields; i++)
-        {
-        if (c + 4 > end)
-            {
-            fprintf (stderr, "Invalid/corrupted comments\n");
-            return -1;
-            }
-        
-        len = readint(c, 0);
-        c += 4;
-        if (c + len > end)
-            {
-            fprintf (stderr, "Invalid/corrupted comments\n");
-            return -1;
-            }
-        
-        if (!strncasecmp(c, match_string, strlen(match_string)))
-            count += 1;
-        c += len;
-        }
-    return count;
-    }
-
-static void speex_get_comments(struct oggdec_vars *self, char *comments, int length)
-    {
-    char *c=comments;
-    int len, i, nb_fields, use_alt;
-    char *end;
-
-    use_alt = speex_comment_count(comments, length, "trk-title") ? TRUE : FALSE;
-    if (use_alt)
-        fprintf(stderr, "using alternative speex tags\n");
-    else
-        fprintf(stderr, "using regular speex tags\n");
-
-    void handle_keyval(char *key, char *val, char **target, char *match, int multiple)
-        {
-        char *old;
-
-        if (!strcasecmp(key, match))
-            {
-            if (*target && *target[0] && multiple)
-                {
-                old = strdup(*target);
-                *target = realloc(*target, strlen(old) + strlen(val) + 2);
-                sprintf(*target, "%s/%s", old, val);
-                free(old);
-                }
-            else
-                {
-                *target = realloc(*target, strlen(val) + 1);
-                strcpy(*target, val);
-                }
-            }
-        }
-  
-    void handle_comment(char *comment, int comment_length)
-        {
-        char *key = malloc(comment_length + 1);
-        char *val;
-        
-        memcpy(key, comment, comment_length);
-        key[comment_length] = '\0';
-        val = strchr(key, '=');
-        if (!val)
-            {
-            fprintf(stderr, "Invalid/corrupted comments\n");
-            free(key);
-            return;
-            }
-        *val++ = '\0';
-
-        if (use_alt)
-            {
-            handle_keyval(key, val, &self->artist[self->ix], "trk-author", TRUE);
-            handle_keyval(key, val, &self->artist[self->ix], "trk-artist", TRUE);
-            handle_keyval(key, val, &self->title[self->ix], "trk-title", TRUE);
-            handle_keyval(key, val, &self->album[self->ix], "trk-album", TRUE);
-            }
-        else
-            {
-            handle_keyval(key, val, &self->artist[self->ix], "author", TRUE);
-            handle_keyval(key, val, &self->artist[self->ix], "artist", TRUE);
-            handle_keyval(key, val, &self->title[self->ix], "title", TRUE);
-            handle_keyval(key, val, &self->album[self->ix], "album", TRUE);
-            }
-        handle_keyval(key, val, &self->replaygain[self->ix], "replaygain_track_gain", FALSE);
-        free(key);
-        }
-    
-    self->artist[self->ix] = NULL;
-    self->title[self->ix] = NULL;
-    self->album[self->ix] = NULL;
-    
-    if (length < 8)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return;
-        }
-    
-    end = c + length;
-    len = readint(c, 0);
-    c += 4;
-    
-    if (c + len > end)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return;
-        }
-
-    c += len;
-    if (c + 4 > end)
-        {
-        fprintf (stderr, "Invalid/corrupted comments\n");
-        return;
-        }
-
-    nb_fields = readint(c, 0);
-    c += 4;
-    for (i = 0; i < nb_fields; i++)
-        {
-        if (c + 4 > end)
-            {
-            fprintf (stderr, "Invalid/corrupted comments\n");
-            return;
-            }
-        
-        len = readint(c, 0);
-        c += 4;
-        if (c + len > end)
-            {
-            fprintf (stderr, "Invalid/corrupted comments\n");
-            return;
-            }
-        
-        handle_comment(c, len);
-        c += len;
-        }
-        
-    if (self->artist[self->ix] == NULL)
-        self->artist[self->ix] = strdup("");
-    if (self->title[self->ix] == NULL)
-        self->title[self->ix] = strdup("");
-    if (self->album[self->ix] == NULL)
-        self->album[self->ix] = strdup("");
-    }
-
 static int speex_get_samplerate(struct oggdec_vars *self)
     {
     SpeexHeader *h;
@@ -609,7 +426,30 @@ static int speex_get_samplerate(struct oggdec_vars *self)
                 speex_header_free(h);
                 if (oggdec_get_next_packet(self) && ogg_stream_packetout(&self->os, &self->op) == 0)
                     {
-                    speex_get_comments(self, (char *)self->op.packet, self->op.bytes);
+                    struct vtag *tag;
+                    int error;
+
+                    if ((tag = vtag_parse((char *)self->op.packet, self->op.bytes, &error)))
+                        {
+                        if (!(self->artist[self->ix] = vtag_lookup(tag, "trk-author", VLM_MERGE, "/")))
+                            if (!(self->artist[self->ix] = vtag_lookup(tag, "trk-artist", VLM_MERGE, "/")))
+                                if (!(self->artist[self->ix] = vtag_lookup(tag, "author", VLM_MERGE, "/")))
+                                    if (!(self->artist[self->ix] = vtag_lookup(tag, "artist", VLM_MERGE, "/")))
+                                        self->artist[self->ix] = strdup("");
+                        if (!(self->title[self->ix] = vtag_lookup(tag, "trk-title", VLM_MERGE, "/")))
+                            if (!(self->title[self->ix] = vtag_lookup(tag, "title", VLM_MERGE, "/")))
+                                self->title[self->ix] = strdup("");
+                        if (!(self->album[self->ix] = vtag_lookup(tag, "trk-album", VLM_MERGE, "/")))
+                            if (!(self->album[self->ix] = vtag_lookup(tag, "album", VLM_MERGE, "/")))
+                                self->album[self->ix] = strdup("");
+
+                        vtag_cleanup(tag);
+                        }
+                    else
+                        {
+                        fprintf(stderr, "%s\n", vtag_strerror(error));
+                        return 0;
+                        }
                     }
                 else
                     return 0;
@@ -627,8 +467,216 @@ static int speex_get_samplerate(struct oggdec_vars *self)
         return 0;
         }
     }
+
 #endif /* HAVE_SPEEX */
 
+#ifdef HAVE_OPUS
+
+static int opus_get_samplerate(struct oggdec_vars *self)
+    {
+    int channels, chanmap, streamcount, streamcount_2c, frames, samples, packetsamples;
+    unsigned initial_granulepos, final_granulepos;
+    uint16_t preskip;
+    char const *reason;
+    
+    #define FAIL(x) do {reason = x; goto fail_point;} while(0)
+    #define WARN(x) do {fprintf(stderr, "opus_get_samplerate: warning: %s\n", x);} while (0)
+
+    if ((final_granulepos = self->final_granulepos[self->ix]) == 0)
+        FAIL("stream final packet granule count is zero");
+
+    if (oggdec_get_next_packet(self) && ogg_stream_packetout(&self->os, &self->op) == 0)
+        {
+        if (ogg_page_granulepos(&self->og) != 0)
+            FAIL("non zero granule position");
+
+        if (ogg_page_packets(&self->og) != 1 || ogg_page_continued(&self->og) || ogg_page_pageno(&self->og) != 0)
+            FAIL("bad header page alignment");
+
+        if (self->op.bytes < 19)
+            FAIL("packet too small to be version 1");
+            
+        if (self->op.packet[8] > 15)
+            FAIL("encapsulation version unsupported");
+            
+        if ((channels = ((unsigned char *)self->op.packet)[9]) == 0)
+            FAIL("number of channels is zero");
+        
+        self->channels[self->ix] = (channels == 1) ? 1 : 2;
+        chanmap = ((unsigned char *)self->op.packet)[18];
+
+        if (chanmap > 1)
+            FAIL("unsupported channel map");
+
+        if ((chanmap == 0 && channels > 2) || (chanmap == 1 && channels > 8))
+            FAIL("too many channels for given channel mapping");
+
+        if (chanmap == 0 && self->op.bytes != 19)
+            FAIL("OpusHead packet size wrong");
+
+        if (chanmap == 1)
+            {
+            if (self->op.bytes != 21 + channels)
+                FAIL("OpusHead packet size wrong");
+            
+            streamcount = ((unsigned char *)self->op.packet)[19];
+            streamcount_2c = ((unsigned char *)self->op.packet)[20];
+            if (streamcount == 0)
+                FAIL("streamcount is zero");
+            if (streamcount_2c > streamcount)
+                FAIL("two channel streamcount > total streamcount");
+            if (streamcount_2c + streamcount > 255)
+                FAIL("combined streamcount quantity exceeds 255");
+                
+            unsigned char *cm = self->op.packet + 21;
+            int index;
+            for (int i = 0; i < channels; ++i)
+                {
+                index = *cm++;
+                if (index != 255 && index >= streamcount + streamcount_2c)
+                    FAIL("bad channel map");
+                }
+            }
+
+        preskip = self->op.packet[10] | (uint16_t)((unsigned char *)self->op.packet)[11] << 8;
+        if (preskip >= final_granulepos)
+            FAIL("no samples to decode after preskip");
+
+        if (oggdec_get_next_packet(self) && ogg_stream_packetout(&self->os, &self->op) == 0)
+            {
+            if (ogg_page_packets(&self->og) != 1 || ogg_page_continued(&self->og) || ogg_page_pageno(&self->og) < 1)
+                FAIL("bad header page alignment");
+
+            if (ogg_page_granulepos(&self->og) != 0)
+                FAIL("non zero granule position");
+
+            if (self->op.bytes >= 8 && !memcmp(self->op.packet, "OpusTags", 8))
+                {
+                struct vtag *tag;
+                int error;
+
+                if ((tag = vtag_parse((char *)self->op.packet + 8, self->op.bytes - 8, &error)))
+                    {
+                    if (!(self->artist[self->ix] = vtag_lookup(tag, "trk-author", VLM_MERGE, "/")))
+                        if (!(self->artist[self->ix] = vtag_lookup(tag, "trk-artist", VLM_MERGE, "/")))
+                            if (!(self->artist[self->ix] = vtag_lookup(tag, "author", VLM_MERGE, "/")))
+                                if (!(self->artist[self->ix] = vtag_lookup(tag, "artist", VLM_MERGE, "/")))
+                                    self->artist[self->ix] = strdup("");
+                    if (!(self->title[self->ix] = vtag_lookup(tag, "trk-title", VLM_MERGE, "/")))
+                        if (!(self->title[self->ix] = vtag_lookup(tag, "title", VLM_MERGE, "/")))
+                            self->title[self->ix] = strdup("");
+                    if (!(self->album[self->ix] = vtag_lookup(tag, "trk-album", VLM_MERGE, "/")))
+                        if (!(self->album[self->ix] = vtag_lookup(tag, "album", VLM_MERGE, "/")))
+                            self->album[self->ix] = strdup("");
+
+                    if (!(self->artist || self->title || self->album))
+                        FAIL("malloc failure");
+
+                    int track_gain_tags = vtag_comment_count(tag, "R128_TRACK_GAIN");
+                    char *track_gain_text = vtag_lookup(tag, "R128_TRACK_GAIN", VLM_FIRST, NULL);
+                    vtag_cleanup(tag);
+                    
+                    switch (track_gain_tags)
+                        {
+                        case 0:
+                            break;
+                        case 1:
+                            if (!track_gain_text)
+                                FAIL("vtag lookup failure");
+                            
+                            if (isspace(track_gain_text[0]))
+                                FAIL("R128_TRACK_GAIN contains whitespace at start");
+
+                            char *endp;
+                            long value = strtol(track_gain_text, &endp, 10);
+                            int endchar = *endp;
+                            free(track_gain_text);
+                            
+                            if (endchar)
+                                FAIL("R128_TRACK_GAIN contains non digit data");
+                                
+                            if (endp - track_gain_text > 6 || value < -32768 || value > 32767)
+                                FAIL("R128_TRACK_GAIN value out of range");
+
+                            break;
+                        default:
+                            free(track_gain_text);
+                            FAIL("too many R128_TRACK_GAIN tags");
+                        }
+                    }
+                else
+                    FAIL(vtag_strerror(error));
+                }
+            else
+                FAIL("bad or missing OpusTags packet");
+            }
+        else
+            FAIL("failed to get OpusTags packet");
+            
+        if (oggdec_get_next_packet(self))
+            {
+            if ((frames = opus_packet_get_nb_frames(self->op.packet, self->op.bytes)) < 1)
+                FAIL("first packet has no frames");
+            samples = packetsamples = opus_packet_get_samples_per_frame(self->op.packet, 48000) * frames;
+
+            while (self->op.granulepos == -1)
+                {
+                oggdec_get_next_packet(self);
+                if ((frames = opus_packet_get_nb_frames(self->op.packet, self->op.bytes)) < 1)
+                    FAIL("packet with no frames detected");
+                
+                samples += opus_packet_get_samples_per_frame(self->op.packet, 48000) * frames;
+                }
+            
+            if (self->op.e_o_s)
+                {
+                if (samples - self->op.granulepos > packetsamples)
+                    WARN("end trimming > size of one packet");
+                }
+            else
+                {
+                if (self->op.granulepos < samples)
+                    FAIL("first page granule position less than number of samples, end of stream not set");
+
+                if ((initial_granulepos = self->initial_granulepos[self->ix] = self->op.granulepos - samples))
+                    {
+                    if (preskip >= final_granulepos - initial_granulepos)
+                        FAIL("no samples to decode after accounting for initial granulepos");
+                    if (initial_granulepos % samples)
+                        WARN("can't assign initial granulepos to a specific page");
+                    else
+                        {
+                        unsigned int pages_missing = initial_granulepos / samples;
+                        
+                        if (ogg_page_pageno(&self->og) - pages_missing != 2)
+                            WARN("opus_get_samplerate: ogg page numbering granulepos mismatch");
+                        else
+                            fprintf(stderr, "there are %u ogg pages missing -- this is normal for a captured stream\n", pages_missing);
+                        }
+                        
+                    if (samples - (final_granulepos - initial_granulepos) % samples > packetsamples)
+                        WARN("end trimming > size of one packet");
+                    }
+                }
+            }
+        else
+            FAIL("failed to get first data packet");
+        }
+    else
+        FAIL("failed to get OpusHead packet");
+
+    return self->samplerate[self->ix] = 48000;  /* Opus always uses this rate */
+
+    #undef FAIL
+    #undef WARN
+
+    fail_point:
+        fprintf(stderr, "opus_get_samplerate: opus header sanity check failed: %s\n", reason);
+        return 0;
+    }
+
+#endif /* HAVE_OPUS */
+
 /* oggscan_eos: perform a binary search on the ogg file for the e_o_s page
  * and log details of the current logical stream when it is found */
 static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_end, int serial, int depth)
@@ -683,7 +731,8 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
             /* make space for data about this logical stream */
             self->n_streams++;
             self->bos_offset = realloc(self->bos_offset, self->n_streams * sizeof (off_t));
-            self->granule_count = realloc(self->granule_count, self->n_streams * sizeof (unsigned));
+            self->initial_granulepos = realloc(self->initial_granulepos, self->n_streams * sizeof (unsigned));
+            self->final_granulepos = realloc(self->final_granulepos, self->n_streams * sizeof (unsigned));
             self->samplerate = realloc(self->samplerate, self->n_streams * sizeof (int));
             self->channels = realloc(self->channels, self->n_streams * sizeof (int));
             self->serial = realloc(self->serial, self->n_streams * sizeof (int));
@@ -698,14 +747,15 @@ static off_t oggscan_eos(struct oggdec_vars *self, off_t offset, off_t offset_en
             self->streamtype = realloc(self->streamtype, self->n_streams * sizeof (enum streamtype_t));
             self->start_time = realloc(self->start_time, self->n_streams * sizeof (double));
             self->duration = realloc(self->duration, self->n_streams * sizeof (double));
-            if (!(self->bos_offset && self->granule_count && self->serial))
+            if (!(self->bos_offset && self->initial_granulepos && self->final_granulepos && self->serial))
                 {
                 fprintf(stderr, "oggscan_eos: malloc failure\n");
                 self->n_streams = 0;
                 return -1;
                 }
           
-            self->granule_count[self->n_streams - 1] = ogg_page_granulepos(&self->og);
+            self->initial_granulepos[self->n_streams - 1] = 0;
+            self->final_granulepos[self->n_streams - 1] = ogg_page_granulepos(&self->og);
             self->serial[self->n_streams - 1] = serial; 
             if (!eos)
                 fprintf(stderr, "oggscan_eos: an unterminated stream was detected\n");
@@ -847,42 +897,43 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
         ogg_stream_pagein(&self->os, &self->og);
         ogg_stream_packetpeek(&self->os, &self->op);
 
-        if (self->op.bytes >= 7 && !memcmp(self->op.packet, "\x01vorbis", 7))
-            self->streamtype[i] = ST_VORBIS;
-        else
-            {
-            if (self->op.bytes >= 5 && !memcmp(self->op.packet, "\x7F""FLAC", 5))
-                self->streamtype[i] = ST_FLAC;
-            else
+        do {
+            if (self->op.bytes >= 7 && !memcmp(self->op.packet, "\x01vorbis", 7))
                 {
-                if (self->op.bytes >= 5 && !memcmp(self->op.packet, "Speex", 5))
-                    self->streamtype[i] = ST_SPEEX;
-                else
-                    self->streamtype[i] = ST_UNHANDLED;
-                }
-            }
- 
-        switch (self->streamtype[i])
-            {
-            case ST_VORBIS:
+                self->streamtype[i] = ST_VORBIS;
                 samplerate = vorbis_get_samplerate(self);
                 break;
-            case ST_FLAC:
+                }
+
 #ifdef HAVE_OGGFLAC
+            if (self->op.bytes >= 5 && !memcmp(self->op.packet, "\x7F""FLAC", 5))
+                {
+                self->streamtype[i] = ST_FLAC;
                 fseeko(self->fp, self->bos_offset[i], SEEK_SET);
                 samplerate = flac_get_samplerate(self);
-#endif
                 break;
-            case ST_SPEEX:
+                }
+#endif /* HAVE_OGGFLAC */
 #ifdef HAVE_SPEEX
+            if (self->op.bytes >= 5 && !memcmp(self->op.packet, "Speex", 5))
+                {
+                self->streamtype[i] = ST_SPEEX;
                 samplerate = speex_get_samplerate(self);
-#endif
                 break;
-            case ST_UNHANDLED:
-            default:
-                fprintf(stderr, "??? unknown stream type ???\n");
+                }
+#endif /* HAVE_SPEEX */
+#ifdef HAVE_OPUS
+            if (self->op.bytes >= 8 && !memcmp(self->op.packet, "OpusHead", 8))
+                {
+                self->streamtype[i] = ST_OPUS;
+                samplerate = opus_get_samplerate(self);
                 break;
-            }
+                }
+#endif /* HAVE_OPUS */
+
+            self->streamtype[i] = ST_UNHANDLED;
+            fprintf(stderr, "??? unhandled ogg stream type ???\n");
+            } while (0);
 
         self->start_time[i] = start_time;
         if (samplerate == 0)
@@ -892,14 +943,15 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
             }
         else
             {
-            start_time += self->duration[i] = self->granule_count[i] / (double)samplerate;
+            start_time += self->duration[i] = (self->final_granulepos[i] - self->initial_granulepos[i]) / (double)samplerate;
             self->total_duration += self->duration[i];
             }
 #if 0
         fprintf(stderr,
             "#####################\n"
             "beginning offset %d\n"
-            "granule_count    %d\n"
+            "initial_granulepos    %d\n"
+            "final_granulepos    %d\n"
             "serial number    %d\n"
             "artist           %s\n"
             "title            %s\n"
@@ -908,7 +960,9 @@ static struct oggdec_vars *oggdecode_get_metadata(char *pathname)
             "channels         %d\n"
             "start time (s)   %lf\n"
             "duration (s)     %lf\n",
-            (int)self->bos_offset[i], self->granule_count[i],
+            (int)self->bos_offset[i],
+            self->initial_granulepos[i],
+            self->final_granulepos[i],
             self->serial[i], self->artist[i], self->title[i], self->album[i], samplerate,
             self->channels[i], self->start_time[i], self->duration[i]);
 #endif
@@ -937,7 +991,8 @@ static void oggdecode_free_metadata(struct oggdec_vars *self)
             }
             
         free(self->bos_offset);
-        free(self->granule_count);
+        free(self->initial_granulepos);
+        free(self->final_granulepos);
         free(self->serial);
         free(self->artist);
         free(self->title);
@@ -997,7 +1052,7 @@ void oggdecode_seek_to_packet(struct oggdec_vars *self)
                     }
                 }
 
-            if ((granulepos = ogg_page_granulepos(&self->og)) >= 0)
+            if ((granulepos = ogg_page_granulepos(&self->og) - self->initial_granulepos[self->ix]) >= 0)
                 break;
             }
 
@@ -1036,6 +1091,11 @@ void oggdecode_dynamic_dispatcher(struct xlplayer *xlplayer)
             case ST_SPEEX:
 #ifdef HAVE_SPEEX
                 success = ogg_speexdec_init(xlplayer);
+#endif
+                break;
+            case ST_OPUS:
+#ifdef HAVE_OPUS
+                success = ogg_opusdec_init(xlplayer);
 #endif
                 break;
             case ST_UNHANDLED:
@@ -1164,7 +1224,8 @@ int oggdecode_get_metainfo(char *pathname, char **artist, char **title, char **a
             *artist = realloc(*artist, 1);
             *title  = realloc(*title, 1);
             *album  = realloc(*album, 1);
-            *artist[0] = *title[0] = *album[0] = '\0';
+            *replaygain = realloc(*replaygain, 1);
+            *artist[0] = *title[0] = *album[0] = *replaygain[0] = '\0';
             }
         else
             {
