@@ -21,7 +21,7 @@
 #include <ogg/ogg.h>
 #include "xlplayer.h"
 
-enum streamtype_t { ST_UNHANDLED, ST_VORBIS, ST_FLAC, ST_SPEEX };
+enum streamtype_t { ST_UNHANDLED, ST_VORBIS, ST_FLAC, ST_SPEEX, ST_OPUS };
 
 struct oggdec_vars
     {
@@ -44,7 +44,8 @@ struct oggdec_vars
     /* stream info */
 
     off_t  *bos_offset;      /* file position where each stream starts */
-    unsigned *granule_count;   /* number of samples in this stream */
+    unsigned *initial_granulepos;
+    unsigned *final_granulepos;
     int    *serial;          /* the ogg serial numbers */
     unsigned *samplerate;    /* sample rate per channel */
     int    *channels;        /* number of audio channels */
