@@ -30,6 +30,7 @@ struct recorder_vars
     {
     char *record_source;
     char *record_folder;
+    char *record_filename;
     char *pause_button;
     char *auto_pause_button;
     };
@@ -110,6 +111,8 @@ struct recorder
     int artist_title_writes;
     pthread_mutex_t artist_title_mutex;
     int new_artist_title;
+    pthread_mutex_t mode_mutex;
+    pthread_cond_t mode_cv;
     };
 
 struct recorder *recorder_init(struct threads_info *ti, int numeric_id);
