@@ -155,9 +155,11 @@ static struct kvpdict kvpdict[] = {
     { "mode",             &ev.mode, NULL },
     { "metadata_mode",    &ev.metadata_mode, NULL },
     { "standard",         &ev.standard, NULL },
-    { "pregain",         &ev.pregain, NULL },
+    { "pregain",          &ev.pregain, NULL },
+    { "postgain",         &ev.postgain, NULL },
     { "quality",          &ev.quality, NULL },
     { "complexity",       &ev.complexity, NULL },
+    { "framesize",        &ev.framesize, NULL },
     { "filename",         &ev.filename, NULL },
     { "offset",           &ev.offset, NULL },
     { "custom_meta",      &ev.custom_meta, NULL },
@@ -181,6 +183,7 @@ static struct kvpdict kvpdict[] = {
     { "icq",              &sv.icq, NULL },
     { "make_public",      &sv.make_public, NULL },
     { "record_source",    &rv.record_source, NULL },        /* recorder_vars */
+    { "record_filename",  &rv.record_filename, NULL },
     { "record_folder",    &rv.record_folder, NULL },
     { "pause_button",     &rv.pause_button, NULL },
     { "command",  &uv.command, NULL},
