@@ -145,6 +145,8 @@ struct xlplayer
     float ls_aud, ls_str;               /* the gain adjusted audio samples */
     float rs_aud, rs_str;
     uint32_t id;                        /* player identity e.g. player 3 = 1 << 3 */
+    pthread_mutex_t command_mutex;      /* lock for command varaible change */
+    pthread_cond_t command_cv;          /* used to wake up idle worker thread */
     };
 
 /* xlplayer_create: create an instance of the player */
@@ -221,7 +223,7 @@ void xlplayer_smoothing_process(struct xlplayer *self);
 void xlplayer_stats(struct xlplayer *self);
 
 /* group process all players from the list */
-void xlplayer_read_start_all(struct xlplayer **list, jack_nframes_t nframes);
+void xlplayer_read_start_all(struct xlplayer **list, jack_nframes_t nframes, struct xlplayer **roster);
 void xlplayer_read_next_all(struct xlplayer **list);
 void xlplayer_levels_all(struct xlplayer **list);
 void xlplayer_buffer_alloc_all(struct xlplayer **list, jack_nframes_t nframes);
