@@ -36,13 +36,11 @@ _ = gettext.translation(FGlobs.package_name, FGlobs.localedir,
 class TestEncoder(object):
     __metaclass__ = ABCMeta
 
-
     @abstractproperty
     def default_bitrate_stereo(self):
         """A guaranteed good bitrate when encoding 2 channels."""
 
         return int()
-        
 
     @abstractproperty
     def default_bitrate_mono(self):
@@ -51,7 +49,6 @@ class TestEncoder(object):
         Ideally the same value as for stereo if possible."""
 
         return int()
-
         
     @abstractproperty
     def default_samplerate(self):
@@ -59,7 +56,6 @@ class TestEncoder(object):
         
         return int()
 
-
     @abstractproperty
     def suggested_samplerates(self):
         """Useful samplerates for user interface dropdown selection.
@@ -69,7 +65,6 @@ class TestEncoder(object):
         
         return tuple()
 
-
     @abstractproperty
     def suggested_bitrates(self):
         """Useful bitrates for user interface dropdown selection.
@@ -79,13 +74,11 @@ class TestEncoder(object):
         
         return tuple()
 
-
     @abstractmethod
     def test(self, channels, samplerate, bitrate):
         return bool()
 
 
-
 class VorbisTestEncoder(TestEncoder):
     class _VORBIS_INFO(ctypes.Structure):
         _fields_ = [("version", ctypes.c_int), 
@@ -101,36 +94,29 @@ class VorbisTestEncoder(TestEncoder):
     _lv = ctypes.CDLL("libvorbis.so.0")
     _lve = ctypes.CDLL("libvorbisenc.so.2")
 
-
     def __init__(self):
         self._vi = self._VORBIS_INFO()
 
-
     @property
     def default_bitrate_stereo(self):
         return 128000
 
-
     @property
     def default_bitrate_mono(self):
         return 128000
 
-
     @property
     def default_samplerate(self):
         return 44100
 
-
     @property
     def suggested_samplerates(self):
         return (48000, 44100, 32000, 22050, 11025)
 
-
     @property
     def suggested_bitrates(self):
         return (192000, 160000, 128000, 112000, 96000, 80000, 64000, 48000, 45000, 32000)
 
-
     def test(self, channels, samplerate, bitrate):
         """Test run these encoder settings with a vorbis encoder.
         
@@ -149,11 +135,9 @@ class VorbisTestEncoder(TestEncoder):
         return error_code == 0
 
 
-
 class EncoderRange(object):
     """Test out the limits of an encoder's settings."""
 
-
     def __init__(self, encoder):
         """An instance of EncoderRange can probe one type of encoder.
         
@@ -165,7 +149,6 @@ class EncoderRange(object):
         self._working_bitrate = {1: encoder.default_bitrate_mono,
                                  2: encoder.default_bitrate_stereo}
 
-
     def _boundary_search(self, variable_span, test):
         """Encoder working boundary value finder.
         
@@ -205,49 +188,42 @@ class EncoderRange(object):
                 span_1 = [span_2[-1], val1] if found is not None \
                                                         else span_2 + [val1]
 
-
     def lowest_bitrate(self, channels, samplerate):
         """Calculate the lowest working bitrate."""
 
         return self._boundary_search([8000, 1000000],
             lambda bitrate: self._test(channels, samplerate, bitrate))
 
-
     def highest_bitrate(self, channels, samplerate):
         """Calculate the highest working bitrate."""
 
         return self._boundary_search([1000000, 8000],
             lambda bitrate: self._test(channels, samplerate, bitrate))
 
-
     def lowest_samplerate(self, channels, bitrate):
         """Calculate the lowest working samplerate."""
 
         return self._boundary_search([4000, 200000],
             lambda samplerate: self._test(channels, samplerate, bitrate))
 
-
     def highest_samplerate(self, channels, bitrate):
         """Calculate the highest working samplerate."""
 
         return self._boundary_search([200000, 4000],
             lambda samplerate: self._test(channels, samplerate, bitrate))
 
-
     def bitrate_bounds(self, channels, samplerate):
         """Lowest and highest working bitrate as a 2 tuple."""
 
         return self.lowest_bitrate(channels, samplerate), \
                                 self.highest_bitrate(channels, samplerate)
 
-
     def samplerate_bounds(self, channels, bitrate):
         """Lowest and highest working samplerate as a 2 tuple."""
         
         return self.lowest_samplerate(channels, bitrate), \
                                 self.highest_samplerate(channels, bitrate)
 
-
     def bounds(self, channels):
         """Find the absolute lowest and highest encoder supported settings.
         
@@ -270,7 +246,6 @@ class EncoderRange(object):
         
         return {"samplerate_bounds": srb, "bitrate_bounds": brb}
 
-
     def good_samplerates(self, channels, bitrate=None):
         """Returns a tuple of standard suggestion values."""
         
@@ -282,7 +257,6 @@ class EncoderRange(object):
         return tuple(x for x in self._encoder.suggested_samplerates if
                                                         lower <= x <= upper)
 
-
     def good_bitrates(self, channels, samplerate=None):
         """Returns a tuple of standard suggestion values."""
 
@@ -295,7 +269,6 @@ class EncoderRange(object):
                                                         lower <= x <= upper)
 
 
-
 def format_collate(specifier):
     """Takes a FormatDropdown or FormatSpin object, obtains the settings."""
     
@@ -309,7 +282,6 @@ def format_collate(specifier):
     return d
 
 
-
 class FormatDropdown(gtk.VBox):
     def __init__(self, prev_object, title, ident, elements, row, tooltip=None):
         """Parameter 'elements' is a tuple of dictionaries.
@@ -356,18 +328,15 @@ class FormatDropdown(gtk.VBox):
         
         self.show_all()
 
-
     def _cell_data_func(self, cell_layout, cell, model, iter):
         dict_ = model.get_value(iter, 0)
         cell.props.text = dict_["display_text"]
         cell.props.sensitive = dict_.get("sensitive", True)
 
-
     def _on_changed(self, combo_box):
         text = combo_box.props.model[combo_box.props.active][0]["display_text"]
         self._fixed.set_text(text)
 
-
     @property
     def next_element_name(self):
         cbp = self._combo_box.props
@@ -376,22 +345,18 @@ class FormatDropdown(gtk.VBox):
         except KeyError:
             return None
 
-
     @property
     def applied(self):
         return self._fixed.props.visible
         
-        
     @property
     def row(self):
         return self._row
 
-
     @property
     def ident(self):
         return self._ident
 
-
     def apply(self):
         cbp = self._combo_box.props
         if cbp.model[cbp.active][0].get("sensitive", True):
@@ -401,18 +366,15 @@ class FormatDropdown(gtk.VBox):
         else:
             return False
         
-        
     def unapply(self):
         self._combo_box.show()
         self._fixed.hide()
 
-
     @property
     def value(self):
         cbp = self._combo_box.props
         return cbp.model[cbp.active][0]["value"]
 
-
     @value.setter
     def value(self, data):
         if not self.applied:
@@ -423,7 +385,6 @@ class FormatDropdown(gtk.VBox):
                     break
 
 
-
 class FormatSpin(gtk.VBox):
     def __init__(self, prev_object, title, ident, elements, row, unit, next_element_name, suggested_values, tooltip=None):
         """Parameter 'elements' is a tuple of dictionaries.
@@ -470,11 +431,9 @@ class FormatSpin(gtk.VBox):
         size_group.add_widget(prev_object.get_children()[0])
         size_group.add_widget(frame)
 
-
     def _on_changed(self, spin_button):
         self._fixed.set_text(str(int(spin_button.props.value)) + self._unit)
 
-
     def _on_populate_popup(self, spin, menu, values):
         mi = gtk.MenuItem(_('Suggested Values'))
         menu.append(mi)
@@ -488,54 +447,44 @@ class FormatSpin(gtk.VBox):
             submenu.append(mi)
             mi.show()
 
-
     def _on_popup_activate(self, menuitem, spin, value):
         spin.set_value(value)
 
-
     @property
     def next_element_name(self):
         return self._next_element_name
 
-
     @property
     def applied(self):
         return self._fixed.props.visible
 
-
     @property
     def row(self):
         return self._row
 
-
     @property
     def ident(self):
         return self._ident
 
-
     def apply(self):
         self._spin_button.hide()
         self._fixed.show()
         return True
-
         
     def unapply(self):
         self._spin_button.show()
         self._fixed.hide()
         
-        
     @property
     def value(self):
         return str(int(self._spin_button.props.value))
 
-
     @value.setter
     def value(self, value):
         if not self.applied:
             self._spin_button.props.value = int(value)
 
 
-
 class FormatPregain(FormatDropdown):
     """Level adjustment for audio before hitting the codec."""
     
@@ -554,7 +503,6 @@ class FormatPregain(FormatDropdown):
             "ReplayGain makes this feature generally unnecessary and the correct setting in that case is 0 dB."))
 
 
-
 class FormatResampleQuality(FormatDropdown):
     """Resample quality."""
     
@@ -566,7 +514,6 @@ class FormatResampleQuality(FormatDropdown):
             _("All of these settings will provide adequate audio quality. The highest setting will preserve more of the original audio bandwidth at the expense of many CPU cycles."))
 
 
-
 class FormatMetadataChoice(FormatDropdown):
     """User can select the metadata encoding format."""
     
@@ -578,7 +525,6 @@ class FormatMetadataChoice(FormatDropdown):
             _("This affects the stream metadata only. Recordings will use UTF-8 for their metadata."))
 
 
-
 class FormatMetadataUTF8(FormatDropdown):
     """User can select whether to have metadata."""
     
@@ -589,7 +535,6 @@ class FormatMetadataUTF8(FormatDropdown):
             _("Choose whether the stream will carry dynamic metadata. In the case of Ogg streams this is important as a great many players can't handle chained Ogg streams which result from the metadata updates."))
 
 
-
 class FormatMetadataLatin1(FormatDropdown):
     """User can select whether to have metadata."""
     
@@ -600,7 +545,6 @@ class FormatMetadataLatin1(FormatDropdown):
             _("Choose whether to send metadata. Recordings will use UTF-8 metadata."))
 
 
-
 class FormatCodecMPEGMP2Mode(FormatDropdown):
     """MP2 modes."""
     
@@ -621,7 +565,6 @@ class FormatCodecMPEGMP2Mode(FormatDropdown):
             _('Joint Stereo is a good choice on streams with low bitrates.'))
 
 
-
 class FormatCodecMPEGMP2ModeStereo(FormatDropdown):
     """MP2 modes. No mono option due to previous bitrate choice."""
     
@@ -633,7 +576,6 @@ class FormatCodecMPEGMP2ModeStereo(FormatDropdown):
             _('Due to the high bitrate selected, this codec will only support stereo.'))
 
 
-
 class FormatCodecMPEGMP2ModeSingle(FormatDropdown):
     """MP2 modes. Only mono available due to previous bitrate choice."""
     
@@ -645,7 +587,6 @@ class FormatCodecMPEGMP2ModeSingle(FormatDropdown):
             _('Due to the low bitrate selected, this codec will only support mono.'))
 
 
-
 class FormatCodecMPEGMP2V1BitRates(FormatDropdown):
     """MP2 MPEG1 bit rates. Some channel modes are restricted by bitrate."""
     
@@ -667,7 +608,6 @@ class FormatCodecMPEGMP2V1BitRates(FormatDropdown):
             dict(display_text="32 kHz", value="32", chain="FormatCodecMPEGMP2ModeSingle")), 0)
 
 
-
 class FormatCodecMPEGMP2V2BitRates(FormatDropdown):
     """MP2 MPEG2 bit rates."""
     
@@ -689,7 +629,6 @@ class FormatCodecMPEGMP2V2BitRates(FormatDropdown):
             dict(display_text="8 kHz", value="8", chain="FormatCodecMPEGMP2Mode")), 0)
 
 
-
 class FormatCodecMPEGMP2V1SampleRates(FormatDropdown):
     """MP2 MPEG1 sample rates."""
     
@@ -700,7 +639,6 @@ class FormatCodecMPEGMP2V1SampleRates(FormatDropdown):
             dict(display_text="32000 Hz", value="32000", chain="FormatCodecMPEGMP2V1BitRates")), 0)
 
 
-
 class FormatCodecMPEGMP2V2SampleRates(FormatDropdown):
     """MP2 MPEG2 sample rates."""
     
@@ -711,7 +649,6 @@ class FormatCodecMPEGMP2V2SampleRates(FormatDropdown):
             dict(display_text="16000 Hz", value="16000", chain="FormatCodecMPEGMP2V2BitRates")), 0)
 
 
-
 class FormatCodecMPEGMP2(FormatDropdown):
     """MP2 standard selection."""
     
@@ -725,7 +662,6 @@ class FormatCodecMPEGMP2(FormatDropdown):
             _('MPEG2 introduced lower samplerate options and corresponding lower bitrates. Choose 2 if those are required.'))
 
 
-
 class FormatCodecMPEGMP3Quality(FormatDropdown):
     """MP3 quality."""
     
@@ -739,7 +675,6 @@ class FormatCodecMPEGMP3Quality(FormatDropdown):
             _('Higher quality costs more in terms of CPU cycles.'))
 
 
-
 class FormatCodecMPEGMP3Mode(FormatDropdown):
     """MP3 modes."""
     
@@ -759,7 +694,6 @@ class FormatCodecMPEGMP3Mode(FormatDropdown):
             _('Joint Stereo is a good choice on streams with low bitrates'))
 
 
-
 class FormatCodecMPEGMP3V1BitRates(FormatDropdown):
     """MP3 MPEG1 bit rates."""
     
@@ -781,7 +715,6 @@ class FormatCodecMPEGMP3V1BitRates(FormatDropdown):
             dict(display_text="32 kHz", value="32", chain="FormatCodecMPEGMP3Mode")), 0)
 
 
-
 class FormatCodecMPEGMP3V2BitRates(FormatDropdown):
     """MP3 MPEG2 and 2.5 bit rates."""
     
@@ -803,7 +736,6 @@ class FormatCodecMPEGMP3V2BitRates(FormatDropdown):
             dict(display_text="8 kHz", value="8", chain="FormatCodecMPEGMP3Mode")), 0)
 
 
-
 class FormatCodecMPEGMP3V1SampleRates(FormatDropdown):
     """MP3 MPEG1 sample rates."""
     
@@ -814,7 +746,6 @@ class FormatCodecMPEGMP3V1SampleRates(FormatDropdown):
             dict(display_text="32000 Hz", value="32000", chain="FormatCodecMPEGMP3V1BitRates")), 0)
 
 
-
 class FormatCodecMPEGMP3V2SampleRates(FormatDropdown):
     """MP3 MPEG2 sample rates."""
     
@@ -825,7 +756,6 @@ class FormatCodecMPEGMP3V2SampleRates(FormatDropdown):
             dict(display_text="16000 Hz", value="16000", chain="FormatCodecMPEGMP3V2BitRates")), 0)
 
 
-
 class FormatCodecMPEGMP3V2_5SampleRates(FormatDropdown):
     """MP3 MPEG2.5 non standard sample rates."""
     
@@ -836,7 +766,6 @@ class FormatCodecMPEGMP3V2_5SampleRates(FormatDropdown):
             dict(display_text="8000 Hz", value="8000", chain="FormatCodecMPEGMP3V2BitRates")), 0)
 
 
-
 class FormatCodecMPEGMP3(FormatDropdown):
     """MP3 standard selection."""
     
@@ -851,7 +780,6 @@ class FormatCodecMPEGMP3(FormatDropdown):
             dict(display_text=_("V 2.5"), value="2.5", chain="FormatCodecMPEGMP3V2_5SampleRates")), 0)
 
 
-
 class FormatCodecSpeexComplexity(FormatDropdown):
     """Speex cpu usage selection."""
     
@@ -862,7 +790,6 @@ class FormatCodecSpeexComplexity(FormatDropdown):
             _('A quality setting that affects how heavily the CPU is used.'))
 
 
-
 class FormatCodecSpeexQuality(FormatDropdown):
     """Speex quality selection."""
     
@@ -873,7 +800,6 @@ class FormatCodecSpeexQuality(FormatDropdown):
             _('The higher this setting, the higher the bitrate.'))
 
 
-
 class FormatCodecSpeexBandwidth(FormatDropdown):
     """Speex bandwidth selection."""
     
@@ -885,7 +811,6 @@ class FormatCodecSpeexBandwidth(FormatDropdown):
             _('Essentially a samplerate setting.'))
 
 
-
 class FormatCodecSpeexMode(FormatDropdown):
     """Speex mode selection."""
     
@@ -895,7 +820,6 @@ class FormatCodecSpeexMode(FormatDropdown):
             dict(display_text=_("Stereo"), value="stereo", chain="FormatCodecSpeexBandwidth")), 0)
 
 
-
 class FormatCodecFLACBits(FormatDropdown):
     """FLAC bit width selection."""
     
@@ -907,11 +831,9 @@ class FormatCodecFLACBits(FormatDropdown):
             _('24 bit records with the highest level of detail. If file size is a concern maybe FLAC is not the right codec.'))
 
 
-
 class FormatCodecVorbisVariability(FormatDropdown):
     """Vorbis bit rate variability."""
     
-    
     def __init__(self, prev_object):
         FormatDropdown.__init__(self, prev_object, _('Variability'), "variability", (
             dict(display_text=_("Constant"), value="0", chain="FormatMetadataUTF8"),
@@ -923,11 +845,9 @@ class FormatCodecVorbisVariability(FormatDropdown):
             _('This control is for enabling variable bitrate on Vorbis streams.'))
 
 
-
 class FormatCodecVorbisBitRate(FormatSpin):
     """Vorbis bit rate selection."""
     
-    
     def __init__(self, prev_object):
         dict_ = format_collate(prev_object)
         channels = 1 if dict_["mode"] == "mono" else 2
@@ -935,15 +855,13 @@ class FormatCodecVorbisBitRate(FormatSpin):
         sr = int(dict_["samplerate"])
         bounds = er.bitrate_bounds(channels, sr)
         FormatSpin.__init__(self, prev_object, _('Bitrate'), "bitrate",
-            (128000,) + bounds + (1, 10), 0, " Hz", "FormatCodecVorbisVariability",
+            (128000,) + bounds + (1, 10), 0, " bps", "FormatCodecVorbisVariability",
             er.good_bitrates(channels, sr))
 
 
-
 class FormatCodecVorbisSampleRate(FormatSpin):
     """Vorbis sample rate selection."""
     
-    
     def __init__(self, prev_object):
         channels = 1 if format_collate(prev_object)["mode"] == "mono" else 2
         er = EncoderRange(VorbisTestEncoder())
@@ -953,7 +871,6 @@ class FormatCodecVorbisSampleRate(FormatSpin):
             er.good_samplerates(channels))
 
 
-
 class FormatCodecFLACSampleRate(FormatSpin):
     """FLAC sample rate selection."""
     
@@ -964,7 +881,6 @@ class FormatCodecFLACSampleRate(FormatSpin):
             (96000, 88200, 48000, 44100))
 
 
-
 class FormatCodecFLACMode(FormatDropdown):
     """Speex mode selection."""
     
@@ -974,7 +890,6 @@ class FormatCodecFLACMode(FormatDropdown):
             dict(display_text=_("Stereo"), value="stereo", default=True, chain="FormatCodecFLACSampleRate")), 0)
 
 
-
 class FormatCodecVorbisMode(FormatDropdown):
     """Vorbis mode selection."""
     
@@ -984,6 +899,79 @@ class FormatCodecVorbisMode(FormatDropdown):
             dict(display_text=_("Stereo"), value="stereo", default=True, chain="FormatCodecVorbisSampleRate")), 0)
 
 
+class FormatCodecOpusPostGain(FormatDropdown):
+    """Level adjustment for audio before hitting the codec."""
+    
+    def __init__(self, prev_object):
+        FormatDropdown.__init__(self, prev_object, _('Postgain'), "postgain", (
+            dict(display_text=_('3.0 dB'), value="768", chain="FormatMetadataUTF8"),
+            dict(display_text=_('2.5 dB'), value="640", chain="FormatMetadataUTF8"),
+            dict(display_text=_('2.0 dB'), value="512", chain="FormatMetadataUTF8"),
+            dict(display_text=_('1.5 dB'), value="384", chain="FormatMetadataUTF8"),
+            dict(display_text=_('1.0 dB'), value="256", chain="FormatMetadataUTF8"),
+            dict(display_text=_('0.5 dB'), value="128", chain="FormatMetadataUTF8"),
+            dict(display_text=_('0 dB'), value="0", default=True, chain="FormatMetadataUTF8"),
+            dict(display_text=_('-0.5 dB'), value="-128", chain="FormatMetadataUTF8"),
+            dict(display_text=_('-1.0 dB'), value="-256", chain="FormatMetadataUTF8"),
+            dict(display_text=_('-1.5 dB'), value="-384", chain="FormatMetadataUTF8"),
+            dict(display_text=_('-2.0 dB'), value="-512", chain="FormatMetadataUTF8"),
+            dict(display_text=_('-2.5 dB'), value="-640", chain="FormatMetadataUTF8"),
+            dict(display_text=_('-3.0 dB'), value="-768", chain="FormatMetadataUTF8")), 1,
+            _("A gain adjustment for the player to apply."))
+
+
+class FormatCodecOpusVariability(FormatDropdown):
+    """Set VBR, CBR, etc."""
+    
+    def __init__(self, prev_object):
+        FormatDropdown.__init__(self, prev_object, _('Variability'), "variability", (
+            dict(display_text=_('CBR *'), value="cbr", default=True, chain="FormatCodecOpusPostGain"),
+            dict(display_text=_('CVBR'), value="cvbr", chain="FormatCodecOpusPostGain"),
+            dict(display_text=_('VBR'), value="vbr", chain="FormatCodecOpusPostGain")), 0,
+            _("Bitrate variability. Actual VBR operation may require a higher frame size."))
+
+
+class FormatCodecOpusFrameSize(FormatDropdown):
+    """Size of Opus frames in milliseconds."""
+    
+    def __init__(self, prev_object):
+        FormatDropdown.__init__(self, prev_object, _('Frame Size'), "framesize", (
+            dict(display_text=_('60 ms'), value="60", chain="FormatCodecOpusVariability"),
+            dict(display_text=_('40 ms'), value="40", chain="FormatCodecOpusVariability"),
+            dict(display_text=_('20 ms'), value="20", default=True, chain="FormatCodecOpusVariability")), 0,
+            _("A higher frame size may sound better on very low bitrates."))
+
+
+class FormatCodecOpusComplexity(FormatDropdown):
+    """Opus cpu usage selection."""
+    
+    def __init__(self, prev_object):
+        FormatDropdown.__init__(self, prev_object, _('CPU'), "complexity", 
+            tuple(dict(display_text=str(x), value=str(x), chain="FormatCodecOpusFrameSize", default=(x==10))
+                                                            for x in range(10, -1, -1)), 0,
+            _('The encoder complexity setting which affects CPU load.'))
+
+
+class FormatCodecOpusBitRate(FormatSpin):
+    """Opus bit rate selection for stereo."""
+    
+    def __init__(self, prev_object):
+        dict_ = format_collate(prev_object)
+        channels = 1 if dict_["mode"] == "mono" else 2
+        bounds = (6 * channels, 256 * channels)
+        FormatSpin.__init__(self, prev_object, _('Bitrate'), "bitrate",
+            ((64, 96)[channels - 1],) + bounds + (1, 10), 0, " kbps", "FormatCodecOpusComplexity",
+            (256 * channels, 128 * channels, 64 * channels, 48 * channels, 32 * channels, 16 * channels))
+
+
+class FormatCodecOpusMode(FormatDropdown):
+    """Opus mode selection."""
+    
+    def __init__(self, prev_object):
+        FormatDropdown.__init__(self, prev_object, _('Mode'), "mode", (
+            dict(display_text=_("Mono"), value="mono", chain="FormatCodecOpusBitRate"),
+            dict(display_text=_("Stereo"), value="stereo", default=True, chain="FormatCodecOpusBitRate")), 0)
+
 
 class FormatCodecXiphOgg(FormatDropdown):
     """Ogg codec selection."""
@@ -992,12 +980,13 @@ class FormatCodecXiphOgg(FormatDropdown):
         FormatDropdown.__init__(self, prev_object, _('Codec'), "codec", (
             dict(display_text=_('Vorbis'), value="vorbis", chain="FormatCodecVorbisMode"),
             dict(display_text=_('FLAC'), value="flac", chain="FormatCodecFLACMode", sensitive=FGlobs.oggflacenabled),
-            dict(display_text=_('Speex'), value="speex", chain="FormatCodecSpeexMode", sensitive=FGlobs.speexenabled)), 0,
+            dict(display_text=_('Speex'), value="speex", chain="FormatCodecSpeexMode", sensitive=FGlobs.speexenabled),
+            dict(display_text=_('Opus'), value="opus", chain="FormatCodecOpusMode", sensitive=FGlobs.opusenabled),
+            
+            ), 0,
             _('Codecs of the Ogg container.'))
 
 
-
-
 class FormatCodecMPEGAACMode(FormatDropdown):
     """AAC mode selection."""
     
@@ -1007,7 +996,6 @@ class FormatCodecMPEGAACMode(FormatDropdown):
             dict(display_text=_("Stereo"), value="stereo", default=True, chain="FormatMetadataUTF8")), 0)
 
 
-
 class FormatCodecMPEGAACModeStereo(FormatDropdown):
     """AAC+ v2 mode selection which is always stereo."""
     
@@ -1017,7 +1005,6 @@ class FormatCodecMPEGAACModeStereo(FormatDropdown):
             dict(display_text=_("Stereo"), value="stereo", default=True, chain="FormatMetadataUTF8")), 0)
 
 
-
 class FormatCodecMPEGAACBitrate(FormatSpin):
     """AAC bit rate selection."""
     
@@ -1058,7 +1045,6 @@ class FormatCodecMPEGAACSamplerate(FormatDropdown):
             dict(display_text=_('7350 Hz'), value="7350", chain="FormatCodecMPEGAACBitrate")), 0)
 
 
-
 class FormatCodecMPEGAACPlusV2Samplerate(FormatDropdown):
     """Sample rates as allowed when using the ADTS header."""
     
@@ -1068,7 +1054,6 @@ class FormatCodecMPEGAACPlusV2Samplerate(FormatDropdown):
             dict(display_text=_('44100 Hz'), value="44100", chain="FormatCodecMPEGAACPlusV2Bitrate", default=True)), 0)
 
 
-
 class FormatCodecMPEG(FormatDropdown):
     """MPEG codec selection."""
 
@@ -1103,7 +1088,6 @@ class FormatCodecMPEG(FormatDropdown):
             tooltip)
 
 
-
 class FormatFamily(FormatDropdown):
     """Gives choice of codec family/container format e.g. Xiph/Ogg or MPEG.
     
@@ -1119,7 +1103,6 @@ class FormatFamily(FormatDropdown):
             _('Codecs have been grouped by standards body and or container format.'))
 
 
-
 class FormatControl(gtk.VBox):
     __gproperties__ = {
         'cap-icecast': (gobject.TYPE_BOOLEAN, 'icecast capable',
@@ -1135,7 +1118,6 @@ class FormatControl(gtk.VBox):
                         0, gobject.PARAM_READABLE)
     }
     
-    
     def __init__(self, send, receive):
         gtk.VBox.__init__(self)
         self.set_border_width(6)
@@ -1211,7 +1193,6 @@ class FormatControl(gtk.VBox):
         # GTK closures seem to be weak references.
         self.__refs = (elem_box,)
 
-
     def _on_apply(self, apply_button, back_button, elem_box):
         if self._current.apply():
             next_element_name = self._current.next_element_name
@@ -1223,7 +1204,6 @@ class FormatControl(gtk.VBox):
                 elem_box[self._current.row].pack_start(self._current, False)
             back_button.set_sensitive(True)
         
-        
     def _on_back(self, back_button, apply_button):
         apply_button.set_sensitive(True)
         if self._current.applied:
@@ -1236,14 +1216,12 @@ class FormatControl(gtk.VBox):
             self._current.unapply()
         back_button.set_sensitive(self._current.prev_object is not None)
 
-
     def _on_test(self, widget):
         if widget.get_active():
             self.start_encoder_rc()
         else:
             self.stop_encoder_rc()
             
-
     _cap_table = {
             "ogg":
             {
@@ -1252,7 +1230,9 @@ class FormatControl(gtk.VBox):
                 "flac":
                     {"shoutcast": False, "icecast": True, "recordable": True},
                 "speex":
-                    {"shoutcast": False, "icecast": True, "recordable": True}
+                    {"shoutcast": False, "icecast": True, "recordable": True},
+                "opus":
+                    {"shoutcast": False, "icecast": True, "recordable": True},
             },
             "mpeg":
             {
@@ -1267,7 +1247,6 @@ class FormatControl(gtk.VBox):
             }
     }
 
-
     def _update_capabilities(self):
         settings = format_collate(self._current)
         dict_ = self._cap_table[settings["family"]][settings["codec"]]
@@ -1283,18 +1262,15 @@ class FormatControl(gtk.VBox):
             
         self.caps_frame.set_sensitive(self._current.applied)
 
-
     def do_get_property(self, property):
         if property.name not in (
                             "cap-icecast", "cap-shoutcast", "cap-recordable"):
             raise AttributeError('unknown property %s' % property.name)
         return getattr(self, "_" + property.name.replace("-", "_"))
 
-
     def marshall(self):
         return json.dumps(format_collate(self._current))
 
-
     def unmarshall(self, data):
         dict_ = json.loads(data)
         unapplied = dict_.get("__unapplied__", None)
@@ -1314,11 +1290,9 @@ class FormatControl(gtk.VBox):
                 if oldcurr.next_element_name is None:
                     break
 
-
     def get_settings(self):
         return format_collate(self._current)
 
-
     def start_encoder_rc(self):
         """Start the encoder (with reference counter)."""
         
@@ -1328,7 +1302,6 @@ class FormatControl(gtk.VBox):
             self._reference_counter = int(self._start_encoder())
         return self._reference_counter > 0
 
-     
     def stop_encoder_rc(self):
         """Stop the encoder (with reference counter)."""
 
@@ -1337,21 +1310,18 @@ class FormatControl(gtk.VBox):
                 self._stop_encoder()
             self._reference_counter -= 1
 
-
     @property
     def running(self):
         """True if the encoder is currently running."""
 
         return self._reference_counter > 0
         
-        
     @property
     def finalised(self):
         """Return whether the encoder settings list is complete."""
 
         return self._current.applied
 
-
     def _start_encoder(self):
         if self._current.applied:
             kvps = [] 
@@ -1370,9 +1340,7 @@ class FormatControl(gtk.VBox):
             print "encoder settings are not finalised"
             return False
 
-
     def _stop_encoder(self):
         self._back_button.set_sensitive(True)
         self._send("command=encoder_stop\n")
         return self._receive() != "failed"
-        
