@@ -20,6 +20,7 @@ import os
 import json
 import gettext
 from abc import ABCMeta, abstractmethod
+from functools import wraps
 
 import gobject
 import gtk
@@ -33,11 +34,19 @@ t = gettext.translation(FGlobs.package_name, FGlobs.localedir, fallback=True)
 _ = t.gettext
 
 
+class NotebookSR(gtk.Notebook):
+    """Add methods so the save/restore scheme does not have to be extended."""
+
+    def get_active(self):
+        return self.get_current_page()
+        
+    def set_active(self, page):
+        self.set_current_page(page)
+
 
 class LEDDict(dict):
     """Dictionary of pixbufs of LEDs."""
-    
-    
+        
     def __init__(self, size=10):
         names = "clear", "red", "green", "yellow"
         filenames = ("led_unlit_clear_border_64x64.png",
@@ -49,11 +58,9 @@ class LEDDict(dict):
                 FGlobs.pkgdatadir / filename, size, size)
 
 
-
 class CellRendererLED(gtk.CellRendererPixbuf):
     """A cell renderer that displays LEDs."""
-    
-    
+        
     __gproperties__ = {
             "active" : (gobject.TYPE_INT, "active", "active",
                             0, 1, 0, gobject.PARAM_WRITABLE),
@@ -61,13 +68,11 @@ class CellRendererLED(gtk.CellRendererPixbuf):
                             "clear", gobject.PARAM_WRITABLE)
     }
 
-                            
     def __init__(self, size=10, actives=("clear", "green")):
         gtk.CellRendererPixbuf.__init__(self)
         self._led = LEDDict(size)
         self._index = [self._led[key] for key in actives] 
 
-          
     def do_set_property(self, prop, value):
         if prop.name == "active":
             item = self._index[value]
@@ -79,7 +84,6 @@ class CellRendererLED(gtk.CellRendererPixbuf):
         gtk.CellRendererPixbuf.set_property(self, "pixbuf", item)
 
 
-
 class CellRendererTime(gtk.CellRendererText):
     """Displays time in days, hours, minutes."""
     
@@ -105,7 +109,6 @@ class CellRendererTime(gtk.CellRendererText):
         gtk.CellRendererText.set_property(self, "text", text)
 
 
-
 class StandardDialog(gtk.Dialog):
     def __init__(self, title, message, stock_item, label_width, modal, markup):
         gtk.Dialog.__init__(self)
@@ -137,7 +140,6 @@ class StandardDialog(gtk.Dialog):
         aa.set_spacing(6)
 
 
-
 class ConfirmationDialog(StandardDialog):
     """This needs to be pulled out since it's generic."""
     
@@ -154,7 +156,6 @@ class ConfirmationDialog(StandardDialog):
         aa.pack_start(self.ok)
 
 
-
 class ErrorMessageDialog(StandardDialog):
     """This needs to be pulled out since it's generic."""
     
@@ -167,11 +168,10 @@ class ErrorMessageDialog(StandardDialog):
         self.get_action_area().add(b)
 
 
-
 def threadslock(f):
     """Function decorator for thread locking timeout callbacks."""
     
-    
+    @wraps(f)
     def newf(*args, **kwargs):
         gtk.gdk.threads_enter()
         try:
@@ -182,7 +182,6 @@ def threadslock(f):
     return newf
 
 
-
 class DefaultEntry(gtk.Entry):
     def __init__(self, default_text, sensitive_override=False):
         gtk.Entry.__init__(self)
@@ -242,10 +241,8 @@ class DefaultEntry(gtk.Entry):
                 pass
 
 
-
 class HistoryEntry(gtk.ComboBoxEntry):
     """Combobox which performs history function."""
-    
 
     def __init__(self, max_size=6, initial_text=("",), store_blank=True):
         self.max_size = max_size
@@ -260,7 +257,6 @@ class HistoryEntry(gtk.ComboBoxEntry):
         cell.props.wrap_width = geo[2] * 2 // 3
         cell.props.wrap_mode = pango.WRAP_CHAR
 
-
     def update_history(self, *args):
         text = self.child.get_text().strip()
         if self.store_blank or text:
@@ -273,43 +269,35 @@ class HistoryEntry(gtk.ComboBoxEntry):
             # History size is kept trimmed.
             if len(self.ls) > self.max_size:
                 del self.ls[-1]
-
     
     def get_text(self):
         return self.child.get_text()
 
-
     def set_text(self, text):
         self.update_history()
         self.child.set_text(text)
-
         
     def get_history(self):
         self.update_history()
         return "\x00".join([row[0] for row in self.ls])
 
-        
     def set_history(self, hist):
         self.ls.clear()
         for text in reversed(hist.split("\x00")):
             self.set_text(text)
 
 
-
 class NamedTreeRowReference(object):
     """Provides named attribute access to gtk.TreeRowReference objects.
     
     This is a virtual base class.
     Virtual method 'get_index_for_name()' must be provided in a subclass.
     """
-    
-    
+
     __metaclass__ = ABCMeta
 
-
     def __init__(self, tree_row_ref):
-        object.__setattr__(self, "_tree_row_ref", tree_row_ref)
-        
+        object.__setattr__(self, "_tree_row_ref", tree_row_ref)        
         
     @abstractmethod
     def get_index_for_name(self, tree_row_ref, name):
@@ -323,7 +311,6 @@ class NamedTreeRowReference(object):
         
         pass
 
-
     def _index_for_name(self, name):
         try:
             return self.get_index_for_name(self._tree_row_ref, name)
@@ -331,28 +318,30 @@ class NamedTreeRowReference(object):
             raise AttributeError("%s has no attribute: %s" %
                                             (repr(self._tree_row_ref), name))
 
+    def __iter__(self):
+        return iter(self._tree_row_ref)
+        
+    def __len__(self):
+        return len(self._tree_row_ref)
 
     def __getitem__(self, path):
         return self._tree_row_ref[path]
         
-        
     def __setitem__(self, path, data):
         self._tree_row_ref[path] = data
 
-
     def __getattr__(self, name):
         return self._tree_row_ref.__getitem__(self._index_for_name(name))
 
-
     def __setattr__(self, name, data):
         self._tree_row_ref[self._index_for_name(name)] = data
 
+NamedTreeRowReference.register(list)
 
 
 class WindowSizeTracker(object):
     """This class will monitor the un-maximized size of a window."""
 
-
     def __init__(self, window, tracking=True):
         self._window = window
         self._is_tracking = tracking
@@ -361,33 +350,26 @@ class WindowSizeTracker(object):
         window.connect("configure-event", self._on_configure_event)
         window.connect("window-state-event", self._on_window_state_event)
         
-        
     def set_tracking(self, tracking):
         self._is_tracking = tracking
         
-        
     def get_tracking(self):
         return self._is_tracking
 
-
     def get_x(self):
         return self._x
 
-
     def get_y(self):
         return self._y
 
-
     def get_max(self):
         return self._max
 
-
     def get_text(self):
         """Marshalling function for save settings."""
         
         return json.dumps((self._x, self._y, self._max))
         
-        
     def set_text(self, s):
         """Unmarshalling function for load settings."""
         
@@ -396,27 +378,23 @@ class WindowSizeTracker(object):
         except StandardError:
             pass
 
-
     def apply(self):
         self._window.unmaximize()
         self._window.resize(self._x, self._y)
         if self._max:
             gobject.idle_add(threadslock(self._window.maximize))
-        
 
     def _on_configure_event(self, widget, event):
         if self._is_tracking and not self._max:
             self._x = event.width
             self._y = event.height
 
-
     def _on_window_state_event(self, widget, event): 
         if self._is_tracking:
             self._max = event.new_window_state & \
                                         gtk.gdk.WINDOW_STATE_MAXIMIZED != 0
 
 
-
 class IconChooserButton(gtk.Button):
     """Imitate a FileChooserButton but specific to image types.
     
@@ -455,7 +433,6 @@ class IconChooserButton(gtk.Button):
         self._label = label
         self.set_filename(dialog.get_filename())
 
-
     def set_filename(self, f):
         try:
             disp = glib.filename_display_name(f)
@@ -472,11 +449,9 @@ class IconChooserButton(gtk.Button):
             self._dialog.set_filename(f)
         self.emit("filename-changed", self._filename)
 
-
     def get_filename(self):
         return self._filename
 
-
     def _cb_clicked(self, button, dialog):
         response = dialog.run()
         if response == gtk.RESPONSE_OK:
@@ -488,13 +463,11 @@ class IconChooserButton(gtk.Button):
             self.set_filename(None)
         dialog.hide()
 
-
     def __getattr__(self, attr):
         if attr in gtk.FileChooser.__dict__:
             return getattr(self._dialog, attr)
         raise AttributeError("%s has no attribute, %s" % (
                                             self, attr))
-        
 
 
 class IconPreviewFileChooserDialog(gtk.FileChooserDialog):
@@ -518,7 +491,6 @@ class IconPreviewFileChooserDialog(gtk.FileChooserDialog):
         self.connect("update-preview", self._cb_update_preview, image)
         vbox.show_all()
         
-        
     def _cb_update_preview(self, dialog, image):
         f = self.get_preview_filename()
         try:
@@ -529,3 +501,165 @@ class IconPreviewFileChooserDialog(gtk.FileChooserDialog):
             active = True
             image.set_from_pixbuf(pb)
         self.set_preview_widget_active(active)
+
+
+class LabelSubst(gtk.Frame):
+    """User interface label substitution widget -- by the user."""
+
+    def __init__(self, heading):
+        gtk.Frame.__init__(self, " %s " % heading)
+        self.vbox = gtk.VBox()
+        self.vbox.set_border_width(2)
+        self.vbox.set_spacing(2)
+        self.add(self.vbox)
+        self.textdict = {}
+        self.activedict = {}
+
+    def add_widget(self, widget, ui_name, default_text):
+        frame = gtk.Frame(" %s " % default_text)
+        frame.set_label_align(0.5, 0.5)
+        frame.set_border_width(3)
+        self.vbox.pack_start(frame)
+        hbox = gtk.HBox()
+        hbox.set_spacing(3)
+        frame.add(hbox)
+        hbox.set_border_width(2)
+        use_supplied = gtk.RadioButton(None, _("Alternative"))
+        use_default = gtk.RadioButton(use_supplied, _('Default'))
+        self.activedict[ui_name + "_use_supplied"] = use_supplied
+        hbox.pack_start(use_default, False)
+        hbox.pack_start(use_supplied, False)
+        entry = gtk.Entry()
+        self.textdict[ui_name + "_text"] = entry
+        hbox.pack_start(entry)
+        
+        if isinstance(widget, gtk.Frame):
+            def set_text(new_text):
+                new_text = new_text.strip()
+                if new_text:
+                    new_text = " %s " % new_text
+                widget.set_label(new_text or None)
+            widget.set_text = set_text
+
+        entry.connect("changed", self.cb_entry_changed, widget, use_supplied)
+        args = default_text, entry, widget
+        use_default.connect("toggled", self.cb_radio_default, *args)
+        use_supplied.connect_object("toggled", self.cb_radio_default,
+                                                            use_default, *args)
+        use_default.set_active(True)
+        
+    def cb_entry_changed(self, entry, widget, use_supplied):
+        if use_supplied.get_active():
+            widget.set_text(entry.get_text())
+        elif entry.has_focus():
+            use_supplied.set_active(True)
+        
+    def cb_radio_default(self, use_default, default_text, entry, widget):
+        if use_default.get_active():
+            widget.set_text(default_text)
+        else:
+            widget.set_text(entry.get_text())
+            entry.grab_focus()
+
+
+class FolderChooserButton(gtk.Button):
+    """Replaces the now-broken gtk.FileChosserButton for folder selection.
+    
+    The old chooser also had some issues with being able to visually select
+    unmounted partitions that resulted in no change from the last valid
+    selection. This button fixes that by dispensing with the drop down list
+    entirely.
+    
+    In order to work properly this button's dialog must be in folder select
+    mode.
+    """
+
+    __gsignals__ = { 'current-folder-changed' : (gobject.SIGNAL_RUN_FIRST,
+        gobject.TYPE_NONE, (gobject.TYPE_STRING,))
+    }
+
+    def __init__(self, dialog=None):
+        gtk.Button.__init__(self)
+        self._current_folder = None
+        self._handler_ids = []
+        hbox = gtk.HBox()
+        hbox.set_spacing(3)
+        self.add(hbox)
+        self._icon = gtk.image_new_from_stock(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_MENU)
+        hbox.pack_start(self._icon, False)
+        # TC: FolderChooserButton text for null -- no directory is set.
+        self._label = gtk.Label(_("(none)"))
+        self._label.set_alignment(0.0, 0.5)
+        self._label.set_ellipsize(pango.ELLIPSIZE_END)
+        hbox.pack_start(self._label)
+        self._label.show()
+        self.set_dialog(dialog)
+        self.connect("clicked", self._on_clicked)
+        self.get_child().show_all()
+
+    def set_dialog(self, dialog):
+        self._disconnect_from_dialog()
+
+        if dialog is None:
+            self._update_visual()
+        else:
+            self._connect_to_dialog(dialog)
+            self.set_current_folder(dialog.get_current_folder())
+
+    def get_dialog(self):
+        return self._dialog
+
+    def get_current_folder(self):
+        return self._dialog and self._current_folder
+        
+    def set_current_folder(self, new_folder):
+        """Call this, not the underlying dialog."""
+        
+        if new_folder is not None:
+            new_folder = new_folder.strip()
+            if new_folder != os.sep:
+                new_folder = new_folder.rstrip(os.sep)
+            
+            if new_folder != self._current_folder:
+                self._dialog.set_current_folder(new_folder)
+                self.emit("current-folder-changed", new_folder)
+
+    def _update_visual(self):
+        folder_name = self.get_current_folder()
+        if not folder_name:
+            folder_name = _("(none)")
+        else:
+            folder_name = os.path.split(folder_name)[1]
+        self._label.set_text(folder_name)
+
+    def _disconnect_from_dialog(self):
+        for hid in self._handler_ids:
+            self._dialog.handler_disconnect(hid)
+        del self._handler_ids[:]
+        self._dialog = None
+
+    def _connect_to_dialog(self, dialog):
+        app = self._handler_ids.append
+        app(dialog.connect("destroy", self._on_dialog_destroy))
+        self._dialog = dialog
+        
+    def _on_dialog_destroy(self, dialog):
+        del self._handler_ids[:]
+        self._dialog = None
+        if not self.flags() & gtk.IN_DESTRUCTION:
+            self._update_visual()
+
+    def _on_clicked(self, button):
+        if self._dialog is not None:
+            self._dialog.set_current_folder(self._current_folder or "")
+            if self._dialog.run() == gtk.RESPONSE_ACCEPT:
+                new_folder = self._dialog.get_current_folder()
+                if new_folder != self._current_folder:
+                    self.emit('current-folder-changed', new_folder)
+            else:
+                self._dialog.set_current_folder(self._current_folder or "")
+            self._dialog.hide()
+
+    def do_current_folder_changed(self, new_folder):
+        self._current_folder = new_folder
+        self._update_visual()
