@@ -1,6 +1,7 @@
 """IRC bots for IDJC."""
 
-#   Copyright (C) 2011 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2011, 2012
+#   Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -16,10 +17,6 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
-
-__all__ = ["IRCPane"]
-
-
 import re
 import json
 import time
@@ -48,12 +45,11 @@ from .gtkstuff import threadslock
 from .utils import string_multireplace
 from .tooltips import set_tip
 
+__all__ = ["IRCPane"]
 
 _ = gettext.translation(FGlobs.package_name, FGlobs.localedir,
                                                         fallback=True).gettext
 
-
-
 XCHAT_COLOR = {
     0:  0xCCCCCCFF,
     1:  0x000000FF,
@@ -73,7 +69,6 @@ XCHAT_COLOR = {
     15: 0x959595FF
 }
 
-
 MESSAGE_CATEGORIES = (
         # TC: IRC message subcategory, triggers on new track announcements.
         _("Track announce"),
@@ -84,37 +79,30 @@ MESSAGE_CATEGORIES = (
         # TC: IRC message subcategory, triggered once at the stream's end.
         _("On stream down"))
 
-
 ASCII_C0 = "".join(chr(x) for x in range(32))
 
-
 CODES_AND_DESCRIPTIONS = zip((u"%r", u"%t", u"%l", u"%s", u"%n", u"%d", u"%u"),
         (_('Artist'), _('Title'), _('Album'), _('Song name'),
          _('DJ name'), _('Description'), _('Listen URL')))
 
 
-
-
 class IRCEntry(gtk.Entry):  # pylint: disable=R0904
     """Specialised IRC text entry widget.
 
     Features pop-up menu and direct control character insertion.
     """
-    
-    
-    _control_keytable = {107:u"\u0003", 98:u"\u0002",
-                        117:u"\u001F", 111:u"\u000F"}
+
+    _control_keytable = {107: u"\u0003", 98: u"\u0002",
+                        117: u"\u001F", 111: u"\u000F"}
 
     def __init__(self, *args, **kwds):
         gtk.Entry.__init__(self, *args, **kwds)
         self.connect("key-press-event", self._on_key_press_event)
         self.connect("populate-popup", self._popup_menu_populate)
 
-
     def _on_key_press_event(self, entry, event):
         """Handle direct insertion of control characters."""
 
-
         if entry.im_context_filter_keypress(event):
             return True
             
@@ -134,11 +122,9 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
                 entry.set_position(cursor + 1)
                 return True
 
-
     def _popup_menu_populate(self, entry, menu):
         """Builds the right click pop-up menu on the IRCEntry widget."""
         
-        
         # TC: Popup menu item for a GTK text entry widget.
         menuitem = gtk.MenuItem(_('Insert Attribute or Colour Code'))
         menu.append(menuitem)
@@ -149,11 +135,9 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
         self._popup_menu_add_substitutions(entry, submenu)
         self._popup_menu_add_colourselectors(entry, submenu)
 
-
     def _popup_menu_add_substitutions(self, entry, submenu):
         """Adder for menu items that insert substitute characters or codes."""
 
-
         def sub(pairs):
             """Build the attribute inserting menu elements."""
 
@@ -170,7 +154,6 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
                 submenu.append(menuitem)
                 menuitem.show()
 
-
         sub(CODES_AND_DESCRIPTIONS)
 
         # Separate data tokens from formatting tokens.
@@ -186,11 +169,9 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
                     # TC: Text formatting style.
                     _('Normal'))))
 
-
     def _popup_menu_add_colourselectors(self, entry, submenu):
         """Adder for menuitems that choose text colour."""
 
-
         for lower, upper in ((0, 7), (8, 15)):
             menuitem = gtk.MenuItem(_("Colours") + " %d-%d" % (lower, upper))
             submenu.append(menuitem)
@@ -229,22 +210,18 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
                 colourmenuitem.show()
             menuitem.show()
 
-
     @staticmethod
     def _on_menu_item_activate(entry, code):
         """Perform relevant character code insertion."""
         
-        
         cursor = entry.get_position()
         entry.insert_text(code, cursor)
         entry.set_position(cursor + len(code))
 
-
     @staticmethod
     def _on_menu_insert_colour_code(entry, code):
         """Insert the colour palette control code."""
         
-        
         cursor = entry.get_position()
         if cursor < 3 or entry.get_text()[cursor - 3] !="\x03":
             # Foreground colour.
@@ -254,7 +231,6 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
             entry.insert_text(unicode(",%02d" % code), cursor)
         entry.set_position(cursor + 3)
 
-
     @staticmethod
     def _on_colour_box_expose(widget):
         """If we are here the mouse is hovering over a colour palette item.
@@ -266,7 +242,6 @@ class IRCEntry(gtk.Entry):  # pylint: disable=R0904
         widget.set_state(gtk.STATE_NORMAL)
 
 
-
 class IRCView(gtk.TextView):  # pylint: disable=R0904
     """A viewer for IRC text.
     
@@ -274,7 +249,6 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
     Variables are substituted for human readable place markers.
     """
     
-
     matches = tuple((a, re.compile(b)) for a, b in (
         ("foreground_background", "\x03[0-9]{1,2},[0-9]{1,2}"),
         ("foreground",  "\x03[0-9]{1,2}(?!=,)"),
@@ -284,10 +258,8 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         ("text", "[^\x00-\x1F]*"),
         ))
 
-
     readable_equiv = tuple((x, "<%s>" % y) for x, y in CODES_AND_DESCRIPTIONS)
 
-
     def __init__(self):
         gtk.TextView.__init__(self)
         self.set_size_request(500, -1)
@@ -297,7 +269,6 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         self._rslt = self._foreground = self._background = None
         self._bold = self._underline = False
 
-
     def set_text(self, text):
         """Apply text to the viewer.
         
@@ -327,7 +298,6 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         self._foreground = self._background = None
         self._bold = self._underline = False
 
-
     @staticmethod
     def _colour_string(code):
         """The colour as a string of format "#rrggbb.
@@ -336,19 +306,16 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         
         return "#%06X" % (XCHAT_COLOR[int(code)] >> 8)
 
-
     def _handle_bold(self):
         """Bold toggle."""
         
         self._bold = not self._bold
 
-
     def _handle_underline(self):
         """Underline toggle."""
         
         self._underline = not self._underline
 
-
     def _handle_foreground(self):
         """Foreground colour setting."""
 
@@ -357,7 +324,6 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         except IndexError:
             self._foreground = None
 
-
     def _handle_foreground_background(self):
         """Foreground and background colour setting."""
         
@@ -367,14 +333,12 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
         except IndexError:
             self._foreground = self._background = None
 
-
     def _handle_normal(self):
         """The normal formatting tag."""
         
         self._bold = self._underline = False
         self._foreground = self._background = None
 
-
     def _handle_text(self):
         """Normal printable text."""
 
@@ -397,11 +361,9 @@ class IRCView(gtk.TextView):  # pylint: disable=R0904
                                                             tag)      
 
 
-
 class EditDialogMixin(object):
     """Mix-in class to convert initial-data-entry dialogs to edit dialogs."""
     
-    
     def __init__(self, orig_data):
         bb = self.get_action_area()
         self.refresh = gtk.Button(gtk.STOCK_REFRESH)
@@ -413,7 +375,6 @@ class EditDialogMixin(object):
         self.delete = gtk.Button(stock=gtk.STOCK_DELETE)
         bb.add(self.delete)
 
-
     def delete_confirmation(self, deleter):
         """Override in subclass to install a confirmation dialog.
         
@@ -423,18 +384,14 @@ class EditDialogMixin(object):
         return deleter
 
 
-
 server_port_adj = gtk.Adjustment(6667.0, 0.0, 65535.0, 1.0, 10.0)
 
 
-
 class ServerDialog(gtk.Dialog):
     """Data entry dialog for adding a new IRC server."""
 
-
     optinfo = _("Optional data entry field for information only.")
-  
-    
+   
     # TC: Tab heading text.
     def __init__(self, title=_("IRC server")):
         gtk.Dialog.__init__(
@@ -455,6 +412,9 @@ class ServerDialog(gtk.Dialog):
         self.username = gtk.Entry()
         self.password = gtk.Entry()
         self.password.set_visibility(False)
+        self.manual_start = gtk.CheckButton(_("Manual start"))
+        set_tip(self.manual_start, 
+                            _('Off when restarting IDJC and off initially.'))
         self.nick1 = gtk.Entry()
         self.nick2 = gtk.Entry()
         self.nick3 = gtk.Entry()
@@ -469,10 +429,9 @@ class ServerDialog(gtk.Dialog):
         image = gtk.image_new_from_stock(
                                         gtk.STOCK_NETWORK, gtk.ICON_SIZE_DIALOG)
         image.set_alignment(0.5, 0)
-        table = gtk.Table(9, 2)
+        table = gtk.Table(10, 2)
         table.set_col_spacings(6)
         table.set_row_spacings(3)
-        table.set_row_spacing(5, 20)
         rvbox = gtk.VBox(True)
         hbox.pack_start(image, False, padding=20)
         hbox.pack_start(table, True)
@@ -485,7 +444,7 @@ class ServerDialog(gtk.Dialog):
                             # TC: TCP/IP port number label.
                             _("Port"), "",
                             _("User name"),
-                            _("Password"),
+                            _("Password"), "",
                             # TC: IRC nickname data entry label.
                             _("Nickname"),
                             # TC: Second choice of IRC nickname.
@@ -497,8 +456,8 @@ class ServerDialog(gtk.Dialog):
                             # TC: The NickServ password.
                             _("NickServ p/w")),
                 (self.network, self.hostname, self.port, self.ssl,
-                 self.username, self.password, self.nick1, self.nick2,
-                 self.nick3, self.realname, self.nickserv))):
+                 self.username, self.password, self.manual_start, self.nick1,
+                 self.nick2, self.nick3, self.realname, self.nickserv))):
             # TC: Tooltip to IRC 'User name' field.
             set_tip(self.username, _("Ideally set this to something even on "
                                 "servers that allow public anonymous access."))
@@ -525,11 +484,11 @@ class ServerDialog(gtk.Dialog):
         
         self.get_content_area().add(hbox)
         
-        
     def as_tuple(self):
         """Data extraction method."""
 
-        return (self.port.get_value(), self.ssl.get_active(),
+        return (self.manual_start.get_active(),
+            self.port.get_value(), self.ssl.get_active(),
             self.network.get_text().strip(), self.hostname.get_text().strip(),
             self.username.get_text().strip(), self.password.get_text().strip(),
             self.nick1.get_text().strip(), self.nick2.get_text().strip(),
@@ -537,16 +496,13 @@ class ServerDialog(gtk.Dialog):
             self.nickserv.get_text().strip())
 
 
-
 class EditServerDialog(ServerDialog, EditDialogMixin):
     """Adds a delete and restore button to the standard server dialog."""
-
     
     def __init__(self, orig_data):
         ServerDialog.__init__(self)
         EditDialogMixin.__init__(self, orig_data)
 
-
     def delete_confirmation(self, deleter):
         def inner(w):
             cd = ConfirmationDialog("", _("<span weight='bold' size='12000'>"
@@ -557,13 +513,12 @@ class EditServerDialog(ServerDialog, EditDialogMixin):
             cd.show_all()
 
         return inner
-        
          
     def from_tuple(self, orig_data):
         """The data restore method."""
-
         
         n = iter(orig_data).next
+        self.manual_start.set_active(n())
         self.port.set_value(n())
         self.ssl.set_active(n())
         self.network.set_text(n())
@@ -577,17 +532,14 @@ class EditServerDialog(ServerDialog, EditDialogMixin):
         self.nickserv.set_text(n())
 
 
-
 message_delay_adj = gtk.Adjustment(10, 0, 30, 1, 10)
 message_offset_adj = gtk.Adjustment(0, 0, 9999, 1, 10)
 message_interval_adj = gtk.Adjustment(600, 60, 9999, 1, 10)
 
 
-            
 class MessageDialog(gtk.Dialog):
     """Base class for a message creation dialog."""
     
-    
     icon = gtk.STOCK_NEW
 
     def __init__(self, title=None):
@@ -644,12 +596,10 @@ class MessageDialog(gtk.Dialog):
         self.get_content_area().add(self.hbox)
         self.channels.grab_focus()
         
-        
     def _from_channels(self):
         text = self.channels.get_text().replace(",", " ").split()
         return ",".join(x for x in text if x)
 
-
     def _pack(self, widgets):
         vbox = gtk.VBox()
         for l, w in widgets:
@@ -662,17 +612,14 @@ class MessageDialog(gtk.Dialog):
             
         self.hbox.pack_start(vbox, False, padding=20)
 
-
     def as_tuple(self):
         """Data extraction method."""
 
         return self._from_channels(), self.message.get_text().strip()
 
 
-
 class EditMessageDialog(MessageDialog, EditDialogMixin):
     """Adds delete and restore buttons to a message creation dialog."""
-
     
     icon = gtk.STOCK_EDIT
 
@@ -680,20 +627,16 @@ class EditMessageDialog(MessageDialog, EditDialogMixin):
         MessageDialog.__init__(self, title)
         EditDialogMixin.__init__(self, orig_data)
         
-        
     def from_tuple(self, orig_data):
         """The data restore method."""
         
-        
         self.channels.set_text(orig_data[0])
         self.message.set_text(orig_data[1])
 
 
-
 class AnnounceMessageDialog(MessageDialog):
     """Adds delay functionality to the message dialog."""
 
-
     # TC: Dialog window title text.
     title = _("IRC track announce")
 
@@ -711,14 +654,12 @@ class AnnounceMessageDialog(MessageDialog):
         "the listener many seconds ahead of the audio.\n\nThis setting will "
         "help synchronise the track change with the message.")) 
         
-        
     def as_tuple(self):
         """Data extraction method."""
         
         return (self.delay.get_value(), ) + MessageDialog.as_tuple(self)
 
 
-
 class EditAnnounceMessageDialog(AnnounceMessageDialog, EditDialogMixin):
     icon = gtk.STOCK_EDIT
     
@@ -726,12 +667,10 @@ class EditAnnounceMessageDialog(AnnounceMessageDialog, EditDialogMixin):
         AnnounceMessageDialog.__init__(self)
         EditDialogMixin.__init__(self, orig_data)
         
-        
     def from_tuple(self, orig_data):
         return (self.delay.set_value(orig_data[0]),
                   self.channels.set_text(orig_data[1]),
                   self.message.set_text(orig_data[2]))
-
     
 
 class TimerMessageDialog(MessageDialog):
@@ -761,7 +700,6 @@ class TimerMessageDialog(MessageDialog):
                                                 ) + MessageDialog.as_tuple(self)
 
 
-
 class EditTimerMessageDialog(TimerMessageDialog, EditDialogMixin):
     icon = gtk.STOCK_EDIT
     
@@ -769,7 +707,6 @@ class EditTimerMessageDialog(TimerMessageDialog, EditDialogMixin):
         TimerMessageDialog.__init__(self)
         EditDialogMixin.__init__(self, orig_data)
         
-        
     def from_tuple(self, orig_data):
         return (self.offset.set_value(orig_data[0]),
                   self.interval.set_value(orig_data[1]),
@@ -777,7 +714,6 @@ class EditTimerMessageDialog(TimerMessageDialog, EditDialogMixin):
                   self.message.set_text(orig_data[3]))
 
 
-
 def glue(f):
     """IRCPane function decorator for new/edit button callbacks.
     
@@ -821,11 +757,9 @@ def glue(f):
     return inner
 
 
-
 def highlight(f):
     """IRCPane function decorator to highlight newly added item."""
     
-    
     @wraps(f)
     def inner(self, mode, model, iter, *args, **kwds):
         new_iter = f(self, mode, model, iter, *args, **kwds)
@@ -839,11 +773,9 @@ def highlight(f):
     return inner
     
     
-    
 class IRCTreeView(gtk.TreeView):
     """A gtk.TreeView that has a tooltip which handles IRC text formatting."""
 
-
     def __init__(self, model=None):
         gtk.TreeView.__init__(self, model)
         self.set_headers_visible(False)
@@ -851,7 +783,6 @@ class IRCTreeView(gtk.TreeView):
         self.connect("query-tooltip", self._on_query_tooltip)
         self.set_has_tooltip(True)
         self.tooltip_coords = (0, 0)
-        
     
     def _on_query_tooltip(self, tv, x, y, kb_mode, tooltip):
         """Display an IRCView tooltip for appropriate data elements."""
@@ -866,14 +797,13 @@ class IRCTreeView(gtk.TreeView):
                 iter = model.get_iter(path[0])
                 mode = model.get_value(iter, 0)
                 if mode in (3, 5, 7, 9):
-                    message = model.get_value(iter, 5)
+                    message = model[model.get_path(iter)].message
                     irc_view = IRCView()
                     irc_view.set_text(message)
                     tooltip.set_custom(irc_view)
                     return True
 
 
-
 class IRCRowReference(NamedTreeRowReference):
     """A gtk.TreeRowReference but with named attributes.
     
@@ -881,19 +811,18 @@ class IRCRowReference(NamedTreeRowReference):
     """
     
     _lookup = {
-        1: {"port":2, "ssl":3, "network":4, "hostname":5, "username":6,
-             "password":7, "nick1":8, "nick2":9, "nick3":10, "realname":11,
-             "nickserv":12, "nick":13},
+        1: {"manual":2, "port":3, "ssl":4, "network":5, "hostname":6, "username":7,
+             "password":8, "nick1":9, "nick2":10, "nick3":11, "realname":12,
+             "nickserv":13, "nick":14},
 
-        3: {"delay":3, "channels":4, "message":5},
+        3: {"delay":4, "channels":5, "message":6},
 
-        5: {"offset":2, "interval":3, "channels":4, "message":5, "issue":13},
+        5: {"offset":3, "interval":4, "channels":5, "message":6, "issue":14},
         
-        7: {"channels":4, "message":5},
+        7: {"channels":5, "message":6},
 
-        9: {"channels":4, "message":5}
+        9: {"channels":5, "message":6}
         }
-    
 
     def get_index_for_name(self, tree_row_ref, name):
         """An abstract method of the base class that performs the lookup."""
@@ -907,21 +836,18 @@ class IRCRowReference(NamedTreeRowReference):
             return self._lookup[data_type][name]
 
 
-
 class IRCTreeStore(gtk.TreeStore):
     """The data storage object."""
 
     @property
     def data_format(self):
-        return (int,) * 4 + (str,) * 10
-
+        return (int, ) * 5 + (str, ) * 10
 
     def __init__(self):
         gtk.TreeStore.__init__(self, *self.data_format)
         self._row_changed_blocked = False
         self.connect_after("row-changed", self._on_row_changed)
 
-
     def path_is_active(self, path):
         """True when this and all parent elements are active."""
 
@@ -932,14 +858,11 @@ class IRCTreeStore(gtk.TreeStore):
 
         return False
 
-
     def row_changed_block(self):
         self._row_changed_blocked = True
         
-        
     def row_changed_unblock(self):
         self._row_changed_blocked = False
-        
 
     def _on_row_changed(self, model, path, iter):
         """This is the very first handler that will be called."""
@@ -947,24 +870,21 @@ class IRCTreeStore(gtk.TreeStore):
         if self._row_changed_blocked:
             self.stop_emission("row-changed")
 
-
     def __getitem__(self, path):
         """Properly wrap the TreeRowReference."""
         
         return IRCRowReference(gtk.TreeStore.__getitem__(self, path))
-        
 
 
 class IRCPane(gtk.VBox):
     """The main user interface."""
     
-    
     def __init__(self):
         gtk.VBox.__init__(self)
         self.set_border_width(8)
         self.set_spacing(3)
         self._treestore = IRCTreeStore()
-        self._treestore.insert(None, 0, (0, 1, 0, 0) + ("", ) * 10)
+        self._treestore.insert(None, 0, (0, 1, 0, 0, 0) + ("", ) * 10)
         self._treeview = IRCTreeView(self._treestore)
         
         col = gtk.TreeViewColumn()
@@ -1016,7 +936,6 @@ class IRCPane(gtk.VBox):
 
         self.show_all()
 
-
     def _m_signature(self):
         """The client data storage signature. 
         
@@ -1025,7 +944,6 @@ class IRCPane(gtk.VBox):
 
         return [x.__name__ for x in self._treestore.data_format]
 
-
     def marshall(self):
         """Convert all our data into a string."""
 
@@ -1036,11 +954,12 @@ class IRCPane(gtk.VBox):
         else:
             return ""
 
-
     def _m_read(self, model, path, iter, store):
-        line = tuple(model[path])
-        store.append((path, line))
+        row = IRCRowReference(model[path])
+        if row.type == 1 and row.active and row.manual:
+            row.active = 0
 
+        store.append((path, list(row)))
 
     def unmarshall(self, data):
         """Set the TreeStore with data from a string."""
@@ -1066,20 +985,21 @@ class IRCPane(gtk.VBox):
             selection.handler_unblock_by_func(self._on_selection_changed)
             selection.select_path(0)
 
-
     def _on_selection_changed(self, selection, edit, new):
         model, iter = selection.get_selected()
-        mode = model.get_value(iter, 0)
-        
-        edit.set_sensitive(mode % 2)
-        new.set_sensitive(not mode % 2)
-        
+        if iter is not None:
+            mode = model.get_value(iter, 0)
+            
+            edit.set_sensitive(mode % 2)
+            new.set_sensitive(not mode % 2)
+        else:
+            edit.set_sensitive(False)
+            new.set_sensitive(False)          
 
     def _on_toggle(self, widget):
         model, iter = self._treeview.get_selection().get_selected()
         model.set_value(iter, 1, not model.get_value(iter, 1))
 
-
     def _cell_data_func(self, column, cell, model, iter):
         """Converts tree data into something viewable.
         
@@ -1108,6 +1028,9 @@ class IRCPane(gtk.VBox):
                 if row.nickserv:
                     # TC: Indicator text: We interact with NickServ.
                     opt.append(_("NICKSERV"))
+                if row.manual:
+                    # TC: Indicator text: Server connection started manually.
+                    opt.append(_("MANUAL"))
                 if opt:
                     text += " " + ", ".join(opt)
             else:
@@ -1149,24 +1072,22 @@ class IRCPane(gtk.VBox):
                             " message category" % MESSAGE_CATEGORIES[mode / 2]
             else:
                 print "unknown message category with numerical code,", mode
-     
-    
+
     @glue
     def _on_edit(self, mode, model, iter, dialog):
         if mode == 1:
-            dialog(EditServerDialog(tuple(model[model.get_path(iter)])[2:13]),
+            dialog(EditServerDialog(tuple(model[model.get_path(iter)])[2:14]),
                                                     self._standard_edit, 2)
         if mode == 3:
             dialog(EditAnnounceMessageDialog(tuple(model[model.get_path(iter)])
-                                                [3:6]), self._standard_edit, 3)
+                                                [4:7]), self._standard_edit, 4)
         if mode == 5:
             dialog(EditTimerMessageDialog(tuple(model[model.get_path(iter)])
-                                                [2:6]), self._standard_edit, 2)
+                                                [3:7]), self._standard_edit, 3)
         if mode in (7, 9):
             title = self._dsu if mode == 7 else self._dsd
             dialog(EditMessageDialog(title, tuple(
-                model[model.get_path(iter)])[4:6]), self._standard_edit, 4)
-
+                model[model.get_path(iter)])[5:7]), self._standard_edit, 5)
 
     def _standard_edit(self, d, model, iter, start):
         model.row_changed_block()
@@ -1175,34 +1096,35 @@ class IRCPane(gtk.VBox):
         model.row_changed_unblock()
         model.row_changed(model.get_path(iter), iter)
 
-
     @highlight
     def _add_server(self, d, model, parent_iter):
-        iter = model.insert(parent_iter, 0, (1, 1) + d.as_tuple() + ("", ))
+        # Check whether row initially needs to be switched off.
+        row = IRCRowReference(list((1, 1) + d.as_tuple() + ("", )))
+        if row.manual:
+            row.active = 0
+        
+        iter = model.insert(parent_iter, 0, row)
 
         # Add the subelements.
         for i, x in enumerate(xrange(2, 2 + len(MESSAGE_CATEGORIES) * 2, 2)):
-            model.insert(iter, i, (x, 1, 0, 0) + ("", ) * 10)
+            model.insert(iter, i, (x, 1, 0, 0, 0) + ("", ) * 10)
 
         return iter
 
-                    
     @highlight
     def _add_announce(self, d, model, parent_iter):
-        return model.insert(parent_iter, 0, (3, 1, 0) + d.as_tuple()
+        return model.insert(parent_iter, 0, (3, 1, 0, 0) + d.as_tuple()
                                                                 + ("", ) * 8)
 
-
     @highlight
     def _add_timer(self, d, model, parent_iter):
-        return model.insert(parent_iter, 0, (5, 1) + d.as_tuple() + ("", ) * 8)
-        
+        return model.insert(parent_iter, 0, (5, 1, 0) + d.as_tuple()
+                                                                + ("", ) * 8)
     
     @highlight
     def _add_message(self, d, model, parent_iter, mode):
-        return model.insert(parent_iter, 0, (mode + 1, 1, 0, 0) + d.as_tuple() +
-                                                                ("", ) * 8)
-
+        return model.insert(parent_iter, 0, (mode + 1, 1, 0, 0, 0) 
+                                                + d.as_tuple() + ("", ) * 8)
 
 
 class ConnectionsController(list):
@@ -1211,7 +1133,6 @@ class ConnectionsController(list):
     As a list it contains the active server connections.
     """
     
-    
     def __init__(self, model):
         self.model = model
         self._ignore_count = 0
@@ -1223,11 +1144,9 @@ class ConnectionsController(list):
         list.__init__(self)
         self._stream_active = False
      
-     
     def cleanup(self):
         for each in self:
             each.cleanup()
-     
 
     def set_stream_active(self, stream_active):
         self._stream_active = stream_active
@@ -1235,17 +1154,14 @@ class ConnectionsController(list):
         for each in self:
             each.set_stream_active(stream_active)
         
-        
     def new_metadata(self, new_meta):
         for each in self:
             each.new_metadata(new_meta)
 
-
     def _on_row_inserted(self, model, path, iter):
         if model.get_value(iter, 0) == 1:
             self.append(IRCConnection(model, path, self._stream_active))
 
-
     def _on_row_deleted(self, model, path):
         if len(path) == 2:
             for i, irc_conn in enumerate(self):
@@ -1254,7 +1170,6 @@ class ConnectionsController(list):
                     del self[i]
                     break
 
-
     def _on_row_changed(self, model, path, iter):
         i = model.iter_children(iter)
         while i is not None:
@@ -1262,11 +1177,9 @@ class ConnectionsController(list):
             i = model.iter_next(i)
 
 
-
 class IRCConnection(gtk.TreeRowReference, threading.Thread):
     """Self explanatory really."""
     
-    
     def __init__(self, model, path, stream_active):
         gtk.TreeRowReference.__init__(self, model, path)
         threading.Thread.__init__(self)
@@ -1286,13 +1199,11 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                                                     self._on_ui_row_changed)))
         self._on_ui_row_changed(model, path, model.get_iter(path))
 
-
     def set_stream_active(self, stream_active):
         self._stream_active = stream_active
         for each in self._message_handlers:
             each.set_stream_active(stream_active)
 
-
     def new_metadata(self, new_meta):
         if self._stream_active:
             self._played.insert(0, (new_meta["songname"], time.time()))
@@ -1301,7 +1212,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         for each in self._message_handlers:
             each.new_metadata(new_meta)
 
-
     def _on_row_inserted(self, model, path, iter):
         if path[:-1] == self.get_path():
             type = model[path].type
@@ -1311,7 +1221,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             mh.connect("privmsg-ready", self._on_privmsg_ready)
             self._message_handlers.append(mh)
 
-
     def _on_channels_changed(self, message_handler, channel_set):
         if self._have_welcome:
             rest = frozenset.union(frozenset(), *(x.props.channels
@@ -1339,12 +1248,10 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             
             self._queue.append(deferred)
 
-
     def _channels_invalidate(self):
         for each in self._message_handlers:
             each.channels_invalidate()
 
-
     def _on_privmsg_ready(self, handler, targets, message, delay):
         if self._have_welcome:
             chan_targets = filter(lambda x: x[0] in "#&", targets)
@@ -1361,7 +1268,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             else:
                 self._queue.append(deferred)
 
-
     def _on_ui_row_changed(self, model, path, iter):
         if path == self.get_path():
             row = self.get_model()[self.get_path()]
@@ -1400,7 +1306,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
 
             self._queue.append(deferred)
 
-        
     def run(self):
         for event in irclib.all_events:
             try:
@@ -1414,7 +1319,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                 self._queue.pop(0)()
             
             self.irc.process_once(0.2)
-            
 
     def cleanup(self):
         def deferred():
@@ -1437,10 +1341,8 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         if self.is_alive():
             self.server.close()
 
-
     def thread_purge_on_disconnect(self, server, event):
         self._keepalive = False
-        
 
     @threadslock
     def _ui_set_nick(self, nickname):
@@ -1449,7 +1351,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             model.row_changed_block()
             model[self.get_path()].nick = nickname
             model.row_changed_unblock()
-            
 
     def _try_alternate_nick(self):
         try:
@@ -1461,7 +1362,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             self._ui_set_nick(nextnick)
             self.server.nick(nextnick)
 
-
     @threadslock
     def _on_welcome(self, server, event):
         print "Got IRC welcome", event.source()
@@ -1482,7 +1382,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         nspw = row.nickserv
         if event.target() != target and nspw:
             self._nick_recover(server, target, nspw)
-            
 
     def _nick_recover(self, server, target, nspw):
         print "Will issue recover and release commands to NickServ"
@@ -1495,7 +1394,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
 
             server.execute_delayed(i, func, args)
 
-
     def _on_privnotice(self, server, event):
         source = event.source()
         if source is not None:
@@ -1520,34 +1418,27 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
                     self._ui_set_nick(event.target())
         else:
             self._generic_handler(server, event)
-        
 
     def _on_disconnect(self, server, event):
         self._have_welcome = False
         self._ui_set_nick("")
         print event.source(), "disconnected"
 
-        
     def _on_nicknameinuse(self, server, event):
         self._try_alternate_nick()
 
-
     def _on_nickcollision(self, server, event):
         self._try_alternate_nick()
-        
-        
+
     def _on_nonicknamegiven(self, server, event):
         self._try_alternate_nick()
         
-        
     def _on_erroneousenickname(self, server, event):
         self._try_alternate_nick()
 
-
     def _on_join(self, server, event):
         print "Channel joined", event.target()
 
-
     def _on_ctcp(self, server, event):
         source = event.source().split("!")[0]
         args = event.arguments()
@@ -1603,11 +1494,9 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
             pass
             #print "CTCP from", source, args
 
-
     def _on_motd(self, server, event):
         pass
 
-
     def _generic_handler(self, server, event):
         return
         print "Type:", event.eventtype()
@@ -1616,7 +1505,6 @@ class IRCConnection(gtk.TreeRowReference, threading.Thread):
         print "Args:", event.arguments()
 
 
-
 class MessageHandler(gobject.GObject):
     __gsignals__ = { 
         'channels-changed': (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_ACTION,
@@ -1636,7 +1524,6 @@ class MessageHandler(gobject.GObject):
     @property
     def stream_active(self):
         return self._stream_active
-    
 
     subst_keys = ("artist", "title", "album", "songname",
                                                 "djname", "description", "url")
@@ -1645,7 +1532,6 @@ class MessageHandler(gobject.GObject):
 
     subst = dict.fromkeys(subst_keys, "<No data>")
 
-
     def __init__(self, model, path, stream_active):
         gobject.GObject.__init__(self)
         self.tree_row_ref = gtk.TreeRowReference(model, path)
@@ -1656,7 +1542,6 @@ class MessageHandler(gobject.GObject):
         model.connect("row-deleted", self.channels_evaluate)
         model.connect_after("row-changed", self.channels_evaluate)
 
-
     def set_stream_active(self, stream_active):
         if self._stream_active != stream_active:
             self._stream_active = stream_active
@@ -1665,30 +1550,24 @@ class MessageHandler(gobject.GObject):
             else:
                 self.on_stream_inactive()
 
-
     def on_stream_active(self):
         pass
         
-        
     def on_stream_inactive(self):
         pass
 
-
     def cleanup(self):
         pass
 
-
     def on_new_metadata(self):
         pass
 
-
     def new_metadata(self, new_meta):
         assert not frozenset(new_meta).difference(frozenset(self.subst_keys))
 
         self.subst.update(new_meta)
         self.on_new_metadata()
         
-        
     def channels_evaluate(self, model, path, iter=None):
         pp = self.tree_row_ref.get_path()
         if path[:-1] == pp:
@@ -1708,15 +1587,12 @@ class MessageHandler(gobject.GObject):
             if nc != self._channels:
                 self.channels_changed(nc)
 
-
     def channels_invalidate(self):
         self._channels = frozenset()
 
-
     def channels_changed(self, new_channels):
         self.emit("channels-changed", new_channels)
 
-
     def do_channels_changed(self, new_channels):
         """Called after the handlers connected on 'channels-changed'.
 
@@ -1724,15 +1600,13 @@ class MessageHandler(gobject.GObject):
         """
         
         self._channels = frozenset(new_channels)
-  
 
     def do_get_property(self, prop):
         if prop.name == 'channels':
             return self._channels
         else:
             raise AttributeError("unknown property '%s'" % prop.name)
-            
-            
+
     def issue_messages(self, delay_calc=lambda row: 0):
         model = self.tree_row_ref.get_model()
         iter = model.get_iter(self.tree_row_ref.get_path())
@@ -1752,39 +1626,33 @@ class MessageHandler(gobject.GObject):
             iter = model.iter_next(iter)
 
 
-
 class MessageHandlerForType_3(MessageHandler):
     def on_new_metadata(self):
         if self.stream_active:
             self.issue_messages(lambda row: row.delay)
 
 
-
 class MessageHandlerForType_5(MessageHandler):
     def __init__(self, *args, **kwargs):
         self._timeout_id = None
         MessageHandler.__init__(self, *args, **kwargs)
         if self.stream_active:
             self.on_stream_active()
-            
-            
+
     def on_stream_active(self):
         self._timeout_id = gobject.timeout_add(500, self._timeout)
-        
-        
+
     def on_stream_inactive(self):
         if self._timeout_id is not None:
             gobject.source_remove(self._timeout_id)
             self._timeout_id = None
-        
-        
+
     @threadslock
     def _timeout(self):
         self.issue_messages(partial(self._delay_calc,
                                                     the_time=int(time.time())))
         return True
         
-        
     def _delay_calc(self, row, the_time):
         """Returns either a delay of 0 or suppression value None."""
         
@@ -1793,19 +1661,16 @@ class MessageHandlerForType_5(MessageHandler):
             row.issue = str(issue)
             return 0
             
-            
     def cleanup(self):
         if self._timeout_id is not None:
             gobject.source_remove(self._timeout_id)
 
 
-
 class MessageHandlerForType_7(MessageHandler):
     def on_stream_active(self):
         self.issue_messages()
 
 
-
 class MessageHandlerForType_9(MessageHandler):
     def on_stream_inactive(self):
         self.issue_messages()
