@@ -22,10 +22,13 @@ import os.path
 import time
 import collections
 import gettext
+import functools
 
 import gobject
 import gtk
 import pango
+import dbus
+import dbus.service
 
 from idjc import FGlobs, PGlobs
 from .gtkstuff import threadslock
@@ -36,13 +39,14 @@ from .tooltips import set_tip
 _ = gettext.translation(FGlobs.package_name, FGlobs.localedir,
                                                         fallback=True).gettext
 
-pm = ProfileManager()
-
+PM = ProfileManager()
 
 
 control_methods= {
     # TC: Control method. Please keep it as Target:Action.
-    'c_tips': _('Prefs enable tooltips'),
+    'c_tips': _('Tooltips enable'),
+    # TC: Control method. Please keep it as Target:Action.
+    'c_sdjmix': _('DJ-mix monitor'),
 
     # TC: Control method. Please keep it as Target:Action.
     'p_pp': _('Player play/pause'),
@@ -117,6 +121,8 @@ control_methods= {
     # TC: Control method. Please keep it as Target:Action.
     'v_vol': _('VoIP set volume'),
     # TC: Control method. Please keep it as Target:Action.
+    'v_mixback': _('VoIP set mixback'),
+    # TC: Control method. Please keep it as Target:Action.
     'v_gain': _('VoIP set gain'),
     # TC: Control method. Please keep it as Target:Action.
     'v_pan': _('VoIP set balance'),
@@ -125,13 +131,11 @@ control_methods= {
     'k_fire': _('Effect play from start'),
 
     # TC: Control method. Please keep it as Target:Action.
-    'j_stop': _('Effects stop all'),
+    'b_stop': _('Effects stop many'),
     # TC: Control method. Please keep it as Target:Action.
-    'j_vol1': _('Effects set volume'),
+    'b_vol1': _('Effects set volume'),
     # TC: Control method. Please keep it as Target:Action.
-    'j_vol2': _('Effects set headroom'),
-    # TC: Control method. Please keep it as Target:Action.
-    'j_ivol': _('Background tracks set volume'),
+    'b_vol2': _('Effects set headroom'),
 
     # TC: Control method. Please keep it as Target:Action.
     's_on': _('Stream set connected'),
@@ -140,7 +144,6 @@ control_methods= {
     'r_on': _('Recorder set recording'),
 }
 
-
 control_targets= {
     'p': _('Player'),
     'm': _('Channel'),
@@ -149,15 +152,19 @@ control_targets= {
     'r': _('Recorder')
 }
 
-
 control_targets_players= (
     _('Left player'),
     _('Right player'),
     _('Background player'),
     _('Focused player'),
-    _('Fadered player'),
+    _('Fadered player')
 )
 
+control_targets_effects_bank= (
+    _('Effects bank 1'),
+    _('Effects bank 2'),
+    _('All effects')
+)
 
 
 class Binding(tuple):
@@ -357,6 +364,8 @@ class Binding(tuple):
         group= self.method[0]
         if group=='p':
             return control_targets_players[self.target]
+        if group=='b':
+            return control_targets_effects_bank[self.target]
         if group in control_targets:
             return '%s %d' % (control_targets[group], self.target+1)
         return ''
@@ -472,6 +481,9 @@ def action_method(*modes):
     return wrap
 
 
+dbusify = functools.partial(dbus.service.method, dbus_interface=PGlobs.dbus_bus_basename)
+
+
 # Controls ___________________________________________________________________
 
 
@@ -526,7 +538,7 @@ class RepeatCache(collections.MutableSet):
             del self._cache[key]
 
 
-class Controls(object):
+class Controls(dbus.service.Object):
     """Dispatch and implementation of input events to action methods.
     """
     # List of controls set up, empty by default. Mapping of input ID to list
@@ -535,6 +547,7 @@ class Controls(object):
     settings= {}
 
     def __init__(self, owner):
+        dbus.service.Object.__init__(self, PM.dbus_bus_name, PGlobs.dbus_objects_basename + "/controls")
         self.owner= owner
         self.learner= None
         self.editing= None
@@ -558,7 +571,7 @@ class Controls(object):
             Binding('k0.ffc7:pk_fire.9.127'),
             Binding('k0.ffc8:pk_fire.a.127'),
             Binding('k0.ffc9:pk_fire.b.127'),
-            Binding('k0.ff1b:pj_stop.b.127'), # Esc stop effects
+            Binding('k0.ff1b:pb_stop.2.127'), # Esc stop effects
             Binding('k0.31:sx_fade.b.0'), # 1-2 xfader sides
             Binding('k0.32:sx_fade.b.127'),
             Binding('k0.63:px_pass.0.127'), # C, pass xfader
@@ -582,7 +595,7 @@ class Controls(object):
     def save_prefs(self, where=None):
         """Store bindings list to prefs file
         """
-        fp= open((where or pm.basedir) / 'controls', 'w')
+        fp= open((where or PM.basedir) / 'controls', 'w')
         for binding in self.bindings:
             fp.write(str(binding)+'\n')
         fp.close()
@@ -590,7 +603,7 @@ class Controls(object):
     def load_prefs(self):
         """Reload bindings list from prefs file
         """
-        cpn = pm.basedir / 'controls'
+        cpn = PM.basedir / 'controls'
         if os.path.isfile(cpn):
             fp= open(cpn)
             self.bindings= []
@@ -694,7 +707,7 @@ class Controls(object):
     # methods in order, so the order they are defined in this code dictates the
     # order they'll appear in in the UI.
 
-    # Preferences
+    # Miscellaneous
     #
     @action_method(Binding.MODE_PULSE, Binding.MODE_SET)
     def c_tips(self, n, v, isd):
@@ -702,6 +715,40 @@ class Controls(object):
         if isd:
             v= 0 if control.get_active() else 127
         control.set_active(v>=64)
+        
+    @dbusify(in_signature='b')
+    def set_enable_tooltips(self, enabled):
+        self.owner.prefs_window.enable_tooltips.set_active(enabled)
+        
+    @dbusify(out_signature='b')
+    def get_enable_tooltips(self):
+        return self.owner.prefs_window.enable_tooltips.get_active()
+
+
+    @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT)
+    def c_sdjmix(self, n, v, isd):
+        active= not self.owner.listen_dj.get_active() if isd else v>=0x40
+        if active:
+            self.owner.listen_dj.set_active(True)
+        else:
+            self.owner.listen_stream.set_active(True)
+
+    @dbusify()
+    def set_listen_dj_mix(self):
+        self.owner.listen_dj.set_active(True)
+        
+    @dbusify(out_signature='b')
+    def get_listen_dj_mix(self):
+        return self.owner.listen_dj.get_active()
+
+    @dbusify()
+    def set_listen_stream_mix(self):
+        self.owner.listen_stream.set_active(True)
+
+    @dbusify(out_signature='b')
+    def get_listen_stream_mix(self):
+        return self.owner.listen_stream.get_active()
+
 
     # Player
     #
@@ -710,29 +757,48 @@ class Controls(object):
         player= self._get_player(n)
         if player is None: return
         is_playing= player.is_playing
-        if not is_playing:
+        if not is_playing and (isd or v >= 0x40):
             player.play.set_active(True)
-        if is_playing if isd else (
-                                not is_playing or player.is_paused)==(v>=0x40):
+        if is_playing if isd else (player.is_paused == (v >= 0x40)):
             player.pause.set_active(not player.pause.get_active())
 
+    @dbusify(in_signature='ubb')
+    def player_playpause(self, index, play, toggle):
+        self.p_pp(index, 127 if play else 0, toggle)
+
+
     @action_method(Binding.MODE_PULSE)
     def p_stop(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
         player.stop.clicked()
 
+    @dbusify(in_signature='u')
+    def player_stop(self, index):
+        self.p_stop(index, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE)
     def p_advance(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
         player.advance()
+        
+    @dbusify(in_signature='u')
+    def player_advance(self, index):
+        self.p_advance(index, 0, 0)
+        
 
     @action_method(Binding.MODE_PULSE)
     def p_prev(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
         player.prev.clicked()
+        
+    @dbusify(in_signature='u')
+    def player_previous(self, index):
+        self.p_prev(index, 0, 0)
+
 
     @action_method(Binding.MODE_PULSE)
     def p_next(self, n, v, isd):
@@ -740,17 +806,32 @@ class Controls(object):
         if player is None: return
         player.next.clicked()
 
+    @dbusify(in_signature='u')
+    def player_next(self, index):
+        self.p_next(index, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE)
     def p_sprev(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
         treeview_selectprevious(player.treeview)
 
+    @dbusify(in_signature='u')
+    def player_select_previous(self, index):
+        self.p_sprev(index, 0, 0)
+        
+
     @action_method(Binding.MODE_PULSE)
     def p_snext(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
         treeview_selectnext(player.treeview)
+        
+    @dbusify(in_signature='u')
+    def player_select_next(self, index):
+        self.p_snext(index, 0, 0)
+
 
     @action_method(Binding.MODE_PULSE)
     def p_sfire(self, n, v, isd):
@@ -758,6 +839,11 @@ class Controls(object):
         if player is None: return
         player.cb_doubleclick(player.treeview, None, None, None)
 
+    @dbusify(in_signature='u')
+    def player_play_selected(self, index):
+        self.p_sfire(index, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def p_stream(self, n, v, isd):
         player= self._get_player(n)
@@ -765,6 +851,11 @@ class Controls(object):
         active= not player.stream.get_active() if isd else v>=0x40
         player.stream.set_active(active)
 
+    @dbusify(in_signature='ubb')
+    def player_set_streammix(self, index, value, toggle):
+        self.p_stream(index, 127 if value else 0, toggle)
+
+
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def p_listen(self, n, v, isd):
         player= self._get_player(n)
@@ -772,10 +863,18 @@ class Controls(object):
         active= not player.listen.get_active() if isd else v>=0x40
         player.listen.set_active(active)
 
+    @dbusify(in_signature='ubb')
+    def player_set_djmix(self, index, value, toggle):
+        self.p_listen(index, 127 if value else 0, toggle)
+
+
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def p_prep(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
+        if player not in (self.owner.player_left, self.owner.player_right):
+            print "player unsupported for this binding"
+            return
         other= self.owner.player_left if player is self.owner.player_right \
                                                 else self.owner.player_right
         prep= player.stream.get_active() if isd else v>=0x40
@@ -799,10 +898,18 @@ class Controls(object):
     def p_vol(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
-        deckadj= self.owner.deck2adj if player is self.owner.player_right \
+        if player.playername in ("left", "right"):
+            deckadj= self.owner.deck2adj if player is self.owner.player_right \
                                                         else self.owner.deckadj
+        elif player.playername == "interlude":
+            deckadj = self.owner.jingles.ivol_adj
         cross= deckadj.get_value()+v if isd else v
         deckadj.set_value(cross)
+        
+    @dbusify(in_signature='uib')
+    def player_set_volume(self, index, value, delta):
+        self.p_vol(index, value, delta)
+
 
     #@action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
     #def p_gain(self, n, v, isd):
@@ -820,10 +927,14 @@ class Controls(object):
     def p_pitch(self, n, v, isd):
         player= self._get_player(n)
         if player is None: return
-        v= v/127.0*24-12
         speed= player.pbspeedbar.get_value()+v if isd else v
         player.pbspeedbar.set_value(speed)
 
+    @dbusify(in_signature='uib')
+    def player_set_pitch(self, index, value, delta):
+        self.p_pitch(index, value, delta)
+
+
     # Playlist methods, to reproduce previous idjcmedia shortcuts
     #
     @action_method(Binding.MODE_PULSE)
@@ -842,6 +953,10 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Stop Control')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_stop_control(self, index):
+        self.p_istop(index, 0, 0)
+
     @action_method(Binding.MODE_PULSE)
     def p_istop2(self, n, v, isd): #s
         player= self._get_player(n)
@@ -850,6 +965,10 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Stop Control 2')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_stop_control_2(self, index):
+        self.p_istop2(index, 0, 0)
+
     @action_method(Binding.MODE_PULSE)
     def p_ianno(self, n, v, isd): #u
         player= self._get_player(n)
@@ -866,6 +985,10 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Transfer Control')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_transfer_control(self, index):
+        self.p_itrans(index, 0, 0)
+
     @action_method(Binding.MODE_PULSE)
     def p_ifade(self, n, v, isd): #f
         player= self._get_player(n)
@@ -874,6 +997,11 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Crossfade Control')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_crossfade_control(self, index):
+        self.p_ifade(index, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE)
     def p_ipitch(self, n, v, isd): #n
         player= self._get_player(n)
@@ -882,6 +1010,11 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Normal Speed Control')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_normal_speed_control(self, index):
+        self.p_ipitch(index, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE)
     def p_igotop(self, n, v, isd):
         player= self._get_player(n)
@@ -890,6 +1023,28 @@ class Controls(object):
                                 player.treeview.get_selection().get_selected()
         player.menuitem_response(None, 'Jump To Top Control')
 
+    @dbusify(in_signature='u')
+    def playlist_insert_jump_to_top_control(self, index):
+        self.p_igotop(index, 0, 0)
+
+
+    @dbusify(in_signature='us')
+    def playlist_insert_pathname(self, index, pathname):
+        player = self._get_player(index)
+        if player is None: return
+        model, iter = player.treeview.get_selection().get_selected()
+        row = player.get_media_metadata(pathname)
+        if row:
+            model.insert_after(iter, row)
+            self.player_select_next(index)
+        
+    @dbusify()
+    def playlist_clear(self, index):
+        player = self._get_player(index)
+        if player is None: return
+        player.liststore.clear()
+
+
     # Both players
     #
     @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
@@ -897,15 +1052,30 @@ class Controls(object):
         v= v/127.0*100
         cross= self.owner.crossadj.get_value()+v if isd else v
         self.owner.crossadj.set_value(cross)
+        
+    @dbusify(in_signature='ib')
+    def crossfade_set(self, value, delta):
+        self.x_fade(0, value, delta) 
+    
 
     @action_method(Binding.MODE_PULSE)
     def x_pass(self, n, v, isd):
         self.owner.passbutton.clicked()
 
+    @dbusify()
+    def crossfade_pass(self):
+        self.x_pass(0, 0, 0)
+
+
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def x_pitch(self, n, v, isd):
         checkbox= self.owner.prefs_window.speed_variance
-        checkbox.set_active(not checkbox.get_active if isd else v>=0x40)
+        checkbox.set_active(not checkbox.get_active() if isd else v>=0x40)
+
+    @dbusify(in_signature='bb')
+    def pitch_enable(self, value, toggle):
+        self.x_pitch(0, 127 if value else 0, toggle)
+
 
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def x_focus(self, n, v, isd):
@@ -919,6 +1089,11 @@ class Controls(object):
                                                         self.owner.player_left
         player.treeview.grab_focus()
 
+    @dbusify(in_signature='ub')
+    def main_player_focus(self, index, toggle):
+        self.x_focus(0, 127 if index else 0, toggle)
+
+
     # Channel
     #
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
@@ -928,10 +1103,26 @@ class Controls(object):
         if button is not None:
             s = not button.get_active() if isd else v>=0x40
             button.set_active(s)
+
+    @dbusify(in_signature='ubb')
+    def channel_open(self, index, value, toggle):
+        self.m_on(index, 127 if value else 0, toggle)
+
         
-    #@action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
-    #def m_vol(self, n, v, isd):
-    #   pass # XXX
+    @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
+    def m_vol(self, n, v, isd):
+        agc = getattr(self.owner.prefs_window, 'mic_control_%d'%n)
+        vol = agc.valuesdict[agc.commandname+'_gain'].get_adjustment()
+        if isd:
+            v += vol.props.value
+        else:
+            v = v / 127.0 * (vol.props.upper - vol.props.lower) + vol.props.lower
+        vol.set_value(v)
+
+    @dbusify(in_signature='uib')
+    def channel_gain(self, index, value, delta):
+        self.m_vol(index, value, delta)
+
 
     #@action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
     #def m_gain(self, n, v, isd):
@@ -945,6 +1136,11 @@ class Controls(object):
         v= pan.get_value()+v if isd else v
         pan.set_value(v)
 
+    @dbusify(in_signature='uib')
+    def channel_pan(self, index, value, delta):
+        self.m_pan(index, value, delta)
+
+
     # VoIP
     #
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
@@ -953,15 +1149,41 @@ class Controls(object):
         s= not phone.get_active() if isd else v>=0x40
         phone.set_active(s)
 
+    @dbusify(in_signature='bb')
+    def voip_mode_public(self, value, toggle):
+        self.v_on(0, 127 if value else 0, toggle)
+
+
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
     def v_prep(self, n, v, isd):
         phone= self.owner.redphone
         s= not phone.get_active() if isd else v>=0x40
         phone.set_active(s)
 
-    #@action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
-    #def v_vol(self, n, v, isd):
-    #   pass # XXX
+    @dbusify(in_signature='bb')
+    def voip_mode_private(self, value, toggle):
+        self.v_prep(0, 127 if value else 0, toggle)
+
+
+    @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
+    def v_vol(self, n, v, isd):
+        vol = self.owner.voipgainadj.get_value() + v if isd else v
+        self.owner.voipgainadj.set_value(vol)
+
+    @dbusify(in_signature='ib')
+    def voip_set_gain(self, value, delta):
+        self.v_vol(0, value, delta)
+
+
+    @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
+    def v_mixback(self, n, v, isd):
+       vol = self.owner.mixbackadj.get_value() + v if isd else v
+       self.owner.mixbackadj.set_value(vol)
+
+    @dbusify(in_signature='ib')
+    def voip_set_mixback_level(self, value, delta):
+        self.v_mixback(0, value, delta)
+    
 
     #@action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
     #def v_gain(self, n, v, isd):
@@ -975,31 +1197,60 @@ class Controls(object):
     #
     @action_method(Binding.MODE_PULSE)
     def k_fire(self, n, v, isd):
-        self.owner.jingles.effects.widgets[n].trigger.clicked()
+        self.owner.jingles.all_effects[n].trigger.clicked()
+
+    @dbusify(in_signature='u')
+    def effect_trigger(self, index):
+        self.k_fire(index, 0, 0)
+
 
     # Jingles player in general
     #
     @action_method(Binding.MODE_PULSE)
-    def j_stop(self, n, v, isd):
-        self.owner.jingles.effects.stop()
+    def b_stop(self, n, v, isd):
+        if n < 2:
+            self.owner.jingles.effect_banks[n].stop()
+        else:
+            banks = self.owner.jingles.effect_banks
+            banks[0].stop()
+            if len(banks) > 1:
+                banks[1].stop()
+
+    @dbusify(in_signature='bb')
+    def effect_bank_stop(self, first, second):
+        if first or second:
+            self.b_stop(2 if first and second else (0, 1)[second], 0, 0)
+
 
     @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
-    def j_vol1(self, n, v, isd):
-        fader= self.owner.jingles.jvol_adj
-        vol= fader.get_value()+v if isd else v
-        fader.set_value(vol)
+    def b_vol1(self, n, v, isd):
+        if n < 2:
+            fader= self.owner.jingles.jvol_adj[n]
+            vol= fader.get_value()+v if isd else v
+            fader.set_value(vol)
+        else:
+            self.b_vol1(0, v, isd)
+            self.b_vol1(1, v, isd)
+
+    @dbusify(in_signature='uib')
+    def effect_bank_gain(self, index, value, delta):
+        self.b_vol1(index, value, delta)
+
 
     @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
-    def j_vol2(self, n, v, isd):
-        fader= self.owner.jingles.jmute_adj
-        vol= fader.get_value()+v if isd else v
-        fader.set_value(vol)
+    def b_vol2(self, n, v, isd):
+        if n < 2:
+            fader= self.owner.jingles.jmute_adj[n]
+            vol= fader.get_value()+v if isd else v
+            fader.set_value(vol)
+        else:
+            self.b_vol2(0, v, isd)
+            self.b_vol2(1, v, isd)
+
+    @dbusify(in_signature='uib')
+    def effect_bank_headroom(self, index, value, delta):
+        self.b_vol2(index, value, delta)
 
-    @action_method(Binding.MODE_DIRECT, Binding.MODE_SET, Binding.MODE_ALTER)
-    def j_ivol(self, n, v, isd):
-        fader= self.owner.jingles.ivol_adj
-        vol= fader.get_value()+v if isd else v
-        fader.set_value(vol)
 
     # Stream connection
     #
@@ -1009,6 +1260,11 @@ class Controls(object):
         s= not connect.get_active() if isd else v>=0x40
         connect.set_active(s)
 
+    @dbusify(in_signature='ubb')
+    def stream_set_connected(self, index, value, toggle):
+        self.s_on(index, 127 if value else 0, toggle)
+
+
     # Recorder
     #
     @action_method(Binding.MODE_PULSE, Binding.MODE_DIRECT, Binding.MODE_SET)
@@ -1020,6 +1276,10 @@ class Controls(object):
         else:
             buttons.stop_button.clicked()
 
+    @dbusify(in_signature='ubb')
+    def recorder_set_recording(self, index, value, toggle):
+        self.r_on(index, 127 if value else 0, toggle)
+
 
 # Generic GTK utilities ______________________________________________________
 
@@ -1206,7 +1466,7 @@ class BindingEditor(gtk.Dialog):
    
     control_method_groups= {
         # TC: binding editor, action pane, first row, toplevel menu.
-        'c': _('Preferences'),
+        'c': _('Miscellaneous'),
         # TC: binding editor, action pane, first row, toplevel menu.
         'p': _('Player'),
         # TC: binding editor, action pane, first row, toplevel menu.
@@ -1218,7 +1478,7 @@ class BindingEditor(gtk.Dialog):
         # TC: binding editor, action pane, first row, toplevel menu.
         'k': _('Single effect'),
         # TC: binding editor, action pane, first row, toplevel menu.
-        'j': _('Jingles players'),
+        'b': _('Effects bank'),
         # TC: binding editor, action pane, first row, toplevel menu.
         's': _('Stream'),
         # TC: binding editor, action pane, first row, toplevel menu.
@@ -1499,6 +1759,8 @@ class BindingEditor(gtk.Dialog):
         group= method[:1]
         if group=='p':
             self.target_field.set_adjustment(PlayerAdjustment())
+        elif group=='b':
+            self.target_field.set_adjustment(EffectsBankAdjustment())
         elif group in 'mksr':
             self.target_field.set_adjustment(TargetAdjustment(group))
         else:
@@ -1641,6 +1903,13 @@ class PlayerAdjustment(CustomAdjustment):
         return control_targets_players.index(text)
     def write_output(self, value):
         return control_targets_players[max(min(int(value), 4), 0)]
+class EffectsBankAdjustment(CustomAdjustment):
+    def __init__(self, value= 0):
+        CustomAdjustment.__init__(self, value, 0, 2, 1)
+    def read_input(self, text):
+        return control_targets_effects_bank.index(text)
+    def write_output(self, value):
+        return control_targets_effects_bank[max(min(int(value), 2), 0)]
 class TargetAdjustment(CustomAdjustment):
     def __init__(self, group, value= 0):
         CustomAdjustment.__init__(self, value, 0, {
