@@ -27,9 +27,9 @@ import gtk
 
 from idjc import FGlobs, PGlobs
 from . import licence_window
-from . import p3db
+from . import songdb
 from . import midicontrols
-from .gtkstuff import WindowSizeTracker
+from .gtkstuff import WindowSizeTracker, DefaultEntry
 from .prelims import ProfileManager
 from .utils import PathStr
 from .tooltips import set_tip, MAIN_TIPS
@@ -118,8 +118,6 @@ class InitialPlayerConfig(gtk.Frame):
 
 
 class AGCControl(gtk.Frame):
-    can_mark = all(hasattr(gtk.Scale, x) for x in ('add_mark', 'clear_marks'))
-
     mic_modes = (
         # TC: Microphone mode combobox text.
         N_('Deactivated'),
@@ -170,6 +168,7 @@ class AGCControl(gtk.Frame):
         hbox.pack_end(sb, False, False, 0)
         hbox.show_all()
         self.valuesdict[self.commandname + "_" + wname] = sb
+        self.fixups.append(lambda: sb.emit("value-changed"))
         return hbox
 
     def frame(self, label, container):
@@ -212,6 +211,7 @@ class AGCControl(gtk.Frame):
         frame.add(ivbox)
         ivbox.show()
         self.activedict[self.commandname + "_" + wname] = cb
+        self.fixups.append(lambda: cb.emit("toggled"))
         return ivbox
     
     def check(self, label_text, wname, save=True):
@@ -221,15 +221,13 @@ class AGCControl(gtk.Frame):
         cb.show()
         if save:
             self.activedict[self.commandname + "_" + wname] = cb
+        self.fixups.append(lambda: cb.emit("toggled"))
         return cb
         
     def cb_open(self, widget):
         active = widget.get_active()
         self.meter.set_led(active)
 
-    def cb_pan_middle(self, button):
-        self.pan.set_value(50)
-
     def cb_mode(self, combobox):
         mode = combobox.get_active()
 
@@ -267,6 +265,7 @@ class AGCControl(gtk.Frame):
         self.valuesdict = {}
         self.activedict = {}
         self.textdict = {}
+        self.fixups = []
         gtk.Frame.__init__(self)
         hbox = gtk.HBox()
         hbox.set_spacing(3)
@@ -299,6 +298,7 @@ class AGCControl(gtk.Frame):
         self.mode_cell = gtk.CellRendererText()
         self.mode.pack_start(self.mode_cell)
         self.mode.set_attributes(self.mode_cell, text=0)
+        self.fixups.append(lambda: self.mode.emit("changed"))
         
         self.vbox.pack_start(self.mode, False, False)
         
@@ -359,6 +359,7 @@ class AGCControl(gtk.Frame):
         self.openaction.connect_proxy(self.open)
         self.open.emit("toggled")
         self.open.set_sensitive(False)
+        self.fixups.append(lambda: self.open.emit("toggled"))
 
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         panframe = gtk.Frame()
@@ -378,6 +379,7 @@ class AGCControl(gtk.Frame):
         self.pan_active.connect("toggled", self.sendnewstats, "pan_active")
         panframe.set_label_widget(hbox)
         hbox.show()
+        self.fixups.append(lambda: self.pan_active.emit("toggled"))
         
         panvbox = gtk.VBox()
         panvbox.set_border_width(1)
@@ -394,21 +396,15 @@ class AGCControl(gtk.Frame):
         self.pan.set_draw_value(False)
         self.pan.connect("value-changed", self.sendnewstats, "pan")
         self.pan.emit("value-changed")
+        self.fixups.append(lambda: self.pan.emit("value-changed"))
         self.valuesdict[self.commandname + "_pan"] = self.pan
         panhbox.pack_start(self.pan)
         r = gtk.Label(_('R'))
         sizegroup.add_widget(r)
         panhbox.pack_start(r, False, False)
-        pancenterbox = gtk.HBox()
-        pancenter = gtk.Button()
-        pancenter.connect("clicked", self.cb_pan_middle)
-        if self.can_mark:
-            self.pan.add_mark(50.0, gtk.POS_BOTTOM, None)
-            self.pan.add_mark(25.0, gtk.POS_BOTTOM, None)
-            self.pan.add_mark(75.0, gtk.POS_BOTTOM, None)
-        else:
-            pancenterbox.pack_start(pancenter, True, False)
-        panvbox.pack_start(pancenterbox, False, False)
+        self.pan.add_mark(50.0, gtk.POS_BOTTOM, None)
+        self.pan.add_mark(25.0, gtk.POS_BOTTOM, None)
+        self.pan.add_mark(75.0, gtk.POS_BOTTOM, None)
         self.vbox.pack_start(panframe, False, False)
         panframe.show_all()
 
@@ -589,7 +585,7 @@ class mixprefs:
         self.parent.mixer_write(string_to_send)
 
 
-    def cb_dj_aud(self, widget):
+    def cb_vol_changed(self, widget):
         self.parent.send_new_mixer_stats()
 
 
@@ -610,6 +606,7 @@ class mixprefs:
                 f.write("[resource_count]\n")
                 for name, widget in self.rrvaluesdict.iteritems():
                     f.write(name + "=" + str(int(widget.get_value())) + "\n")
+                f.write("num_effects=%d\n" % (24 if self.more_effects.get_active() else 12))
         except IOError:
             print "Error while writing out player defaults"
 
@@ -622,13 +619,16 @@ class mixprefs:
                 for name, widget in self.valuesdict.iteritems():
                     f.write(name + "=" + str(widget.get_value()) + "\n")
                 for name, widget in self.textdict.iteritems():
-                    f.write(name + "=" + widget.get_text() + "\n")
+                    if widget.get_text() is not None:
+                        f.write(name + "=" + widget.get_text() + "\n")
+                    else:
+                        f.write(name + "=\n")
         except IOError:
             print "Error while writing out player defaults"
 
             
     def load_player_prefs(self):
-        proktogglevalue = False
+        songdb_active = False
         try:
             file = open(pm.basedir / "playerdefaults")
             
@@ -648,8 +648,8 @@ class mixprefs:
                         value = False
                     else:
                         value = int(value)
-                    if key == "proktoggle":
-                        proktogglevalue = value
+                    if key == "songdb_active":
+                        songdb_active = value
                     else:
                         self.activedict[key].set_active(value)
                 elif self.valuesdict.has_key(key):
@@ -659,8 +659,8 @@ class mixprefs:
             file.close()
         except IOError:
             print "Failed to read playerdefaults file"
-        if proktogglevalue:
-            self.activedict["proktoggle"].set_active(True)
+        if songdb_active:
+            self.activedict["songdb_active"].set_active(songdb_active)
         self.parent.send_new_mixer_stats()
             
     def apply_player_prefs(self):
@@ -716,13 +716,16 @@ class mixprefs:
                           
     def cb_pbspeed(self, widget):
         if widget.get_active():
-            self.parent.player_left.pbspeedbar.set_value(0)
-            self.parent.player_right.pbspeedbar.set_value(0)
+            self.parent.player_left.pbspeedbar.set_value(64.0)
+            self.parent.player_right.pbspeedbar.set_value(64.0)
             self.parent.player_left.pbspeedbox.show()
             self.parent.player_right.pbspeedbox.show()
+            self.parent.jingles.interlude.pbspeedbar.set_value(64.0)
+            self.parent.jingles.interlude.pbspeedbox.show()
         else:
             self.parent.player_left.pbspeedbox.hide()
             self.parent.player_right.pbspeedbox.hide()
+            self.parent.jingles.interlude.pbspeedbox.hide()
         self.parent.send_new_mixer_stats()
 
     def cb_dual_volume(self, widget):
@@ -762,7 +765,16 @@ class mixprefs:
     def show_about(self):
         self.notebook.set_current_page(self.notebook.page_num(self.aboutframe))
         self.window.present()
-            
+
+    def fixup_mic_controls(self):
+        """Send mic preferences to the backend.
+        
+        This needs to be called whenever the backend is restarted.
+        """
+        for mic in self.mic_controls:
+            for fixup in mic.fixups:
+                fixup()
+
     def __init__(self, parent):
         self.parent = parent
         self.parent.prefs_window = self
@@ -790,6 +802,94 @@ class mixprefs:
         generalwindow.add_with_viewport(outervbox)
         generalwindow.show()
         outervbox.set_border_width(3)
+
+        aud_rs_hbox = gtk.HBox()
+        
+        # User can use this to set the audio level in the headphones
+        
+        # TC: The DJ's sound level controller.
+        frame = gtk.Frame(" %s " % _('DJ Audio Level'))
+        frame.set_label_align(0.5, 0.5)
+        frame.set_border_width(3)
+        hbox = gtk.HBox()
+        hbox.set_border_width(5)
+        frame.add(hbox)
+        hbox.show()
+        self.dj_aud_adj = gtk.Adjustment(0.0, -60.0, 0.0, 0.5, 1.0)
+        dj_aud = gtk.SpinButton(self.dj_aud_adj, 1, 1)
+        dj_aud.connect("value-changed", self.cb_vol_changed)
+        hbox.pack_start(dj_aud, True, False, 0)
+        dj_aud.show()
+        set_tip(dj_aud, _('This adjusts the sound level of the DJ audio.'))
+        aud_rs_hbox.pack_start(frame, False, False, 0)
+        frame.show()
+
+        # TC: The alarm sound level.
+        frame = gtk.Frame(" %s " % _('Alarm Level'))
+        frame.set_label_align(0.5, 0.5)
+        frame.set_border_width(3)
+        hbox = gtk.HBox()
+        hbox.set_border_width(5)
+        frame.add(hbox)
+        hbox.show()
+        self.alarm_aud_adj = gtk.Adjustment(0.0, -60.0, 0.0, 0.5, 1.0)
+        alarm_aud = gtk.SpinButton(self.alarm_aud_adj, 1, 1)
+        alarm_aud.connect("value-changed", self.cb_vol_changed)
+        hbox.pack_start(alarm_aud, True, False, 0)
+        alarm_aud.show()
+        set_tip(alarm_aud, _('This adjusts the sound level of the DJ alarm. '
+        'Typically this should be set close to the dj audio level when using the \'%s\''
+        ' feature, otherwise a bit louder.' % _('Music Loudness Compensation')))
+        aud_rs_hbox.pack_start(frame, False, False, 0)
+        frame.show()
+
+        # User can use this to set the resampled sound quality
+        
+        frame = gtk.Frame(" %s " % _('Player Resample Quality'))
+        frame.set_label_align(0.5, 0.5)
+        frame.set_border_width(3)
+        hbox = gtk.HBox()
+        hbox.set_border_width(5)
+        set_tip(hbox,
+        _('This adjusts the quality of the audio resampling method '
+        'used whenever the sample rate of the music file currently playing does'
+        ' not match the sample rate of the JACK sound server. Best mode '
+        'offers the best sound quality but also uses the most CPU (not '
+        'recommended for systems built before 2006). All these modes provide '
+        'adequate sound quality.'))
+        frame.add(hbox)
+        hbox.show()
+        self.best_quality_resample = gtk.RadioButton(None, _('Best'))
+        self.best_quality_resample.connect(
+                                        "toggled", self.cb_resample_quality, 0)
+        rsbox = gtk.HBox()
+        rsbox.pack_start(self.best_quality_resample, True, False, 0)
+        rsbox.show()
+        hbox.add(rsbox)
+        self.best_quality_resample.show()
+        self.good_quality_resample = gtk.RadioButton(
+                                        self.best_quality_resample, _('Medium'))
+        self.good_quality_resample.connect(
+                                        "toggled", self.cb_resample_quality, 1) 
+        rsbox = gtk.HBox()
+        rsbox.pack_start(self.good_quality_resample, True, False, 0)
+        rsbox.show()
+        hbox.add(rsbox)
+        self.good_quality_resample.show()
+        self.fast_resample = gtk.RadioButton(
+                                        self.good_quality_resample, _('Fast'))
+        self.fast_resample.connect("toggled", self.cb_resample_quality, 2) 
+        rsbox = gtk.HBox()
+        rsbox.pack_start(self.fast_resample, True, False, 0)
+        rsbox.show()
+        hbox.add(rsbox)
+        self.fast_resample.show()
+
+        aud_rs_hbox.pack_start(frame, True, True, 0)
+        frame.show()
+        
+        outervbox.pack_start(aud_rs_hbox, False, False, 0)
+        aud_rs_hbox.show()
         
         # TC: the set of features - section heading.
         featuresframe = gtk.Frame(" %s " % _('Feature Set'))
@@ -864,11 +964,21 @@ class mixprefs:
             hbox.show()
             return hbox
 
+        self.more_effects = gtk.RadioButton(None, 
+                                        _('Reserve 24 sound effects slots'))
+        fewer_effects = gtk.RadioButton(self.more_effects, _("Only 12"))
+        if PGlobs.num_effects == 24:
+            self.more_effects.clicked()
+        else:
+            fewer_effects.clicked()
+       
+        rrvbox.pack_start(hjoin(self.more_effects, fewer_effects))
+
         self.mic_qty_adj = gtk.Adjustment(
                                         PGlobs.num_micpairs * 2, 2.0, 12.0, 2.0)
         spin = gtk.SpinButton(self.mic_qty_adj)
         rrvbox.pack_start(hjoin(spin, gtk.Label(
-                                        _('General purpose audio channels*'))))
+                                        _('Audio input channels'))))
     
         self.stream_qty_adj = gtk.Adjustment(
                                             PGlobs.num_streamers, 1.0, 9.0, 1.0)
@@ -881,11 +991,6 @@ class mixprefs:
         rrvbox.pack_start(hjoin(spin, gtk.Label(
                                             _('Simultaneous recording(s)'))))
         
-        # TC: star marked items are relevant only in 'fully featured' mode.
-        key_label = gtk.Label(_("* In 'Fully Featured' mode."))
-        rrvbox.pack_start(key_label)
-        key_label.show()
-
         self.rrvaluesdict = {"num_micpairs": self.mic_qty_adj,
                                     "num_streamers": self.stream_qty_adj,
                                     "num_recorders": self.recorder_qty_adj}
@@ -897,26 +1002,44 @@ class mixprefs:
                 target.show()
             else:
                 target.hide()
-        frame = gtk.Frame(" %s " % _('Meters'))
+        frame = gtk.Frame(" %s " % _('View'))
         frame.set_border_width(3)
+        hbox = gtk.HBox(3, True)
+        hbox.set_border_width(10)
+        frame.add(hbox)
+        hbox.show()
+
         vbox = gtk.VBox()
-        vbox.set_border_width(10)
-        frame.add(vbox)
+        hbox.pack_start(vbox)
         vbox.show()
         self.show_stream_meters = gtk.CheckButton()
         self.show_stream_meters.set_active(True)
         self.show_stream_meters.connect(
                                     "toggled", showhide, parent.streammeterbox)
-        vbox.pack_start(self.show_stream_meters, False, False)
+        vbox.pack_start(self.show_stream_meters, False)
         self.show_stream_meters.show()
+
+        self.show_background_tracks_player = gtk.CheckButton()
+        self.show_background_tracks_player.set_active(True)
+        self.show_background_tracks_player.connect(
+                            "toggled", showhide, parent.jingles.interlude_frame)
+        vbox.pack_start(self.show_background_tracks_player, False)
+        self.show_background_tracks_player.show()
         
-        hbox = gtk.HBox()
-        vbox.pack_start(hbox, False, False)
-        hbox.show()
+        self.show_button_bar = gtk.CheckButton()
+        self.show_button_bar.set_active(True)
+        self.show_button_bar.connect("toggled", showhide, parent.hbox10)
+        self.show_button_bar.connect("toggled", showhide, parent.hbox10spc)
+        vbox.pack_start(self.show_button_bar, False)
+        self.show_button_bar.show()
+
+        vbox = gtk.VBox()
+        hbox.pack_start(vbox)
+        vbox.show()
         self.show_microphones = gtk.CheckButton()
         self.show_microphones.set_active(True)
         self.show_microphones.connect("toggled", showhide, parent.micmeterbox)
-        hbox.pack_start(self.show_microphones, False, False)
+        vbox.pack_start(self.show_microphones, False)
         self.show_microphones.show()                
         
         self.no_mic_void_space = gtk.CheckButton(
@@ -924,7 +1047,7 @@ class mixprefs:
         self.no_mic_void_space.set_active(True)
         for meter in parent.mic_meters:
             self.no_mic_void_space.connect("toggled", meter.always_show)
-        hbox.pack_end(self.no_mic_void_space, False)
+        vbox.pack_start(self.no_mic_void_space, False)
         self.no_mic_void_space.show()
         
         outervbox.pack_start(frame, False, False, 0)
@@ -932,7 +1055,7 @@ class mixprefs:
         
         # ReplayGain controls
         
-        frame = gtk.Frame(" %s " % _('ReplayGain'))
+        frame = gtk.Frame(" %s " % _('Player Loudness Normalisation'))
         frame.set_border_width(3)
         outervbox.pack_start(frame, False, False, 0)
         vbox = gtk.VBox()
@@ -943,55 +1066,79 @@ class mixprefs:
         vbox.show()
         
         self.rg_indicate = gtk.CheckButton(
-                            _('Indicate which tracks have ReplayGain values'))
-        set_tip(self.rg_indicate,
-                        _('Show a marker in the playlists next to each track.'))
+                            _('Indicate which tracks have loudness metadata'))
+        set_tip(self.rg_indicate, _('Shows a marker in the playlists next to'
+                    ' each track. Either a green circle or a red triangle.'))
         self.rg_indicate.connect("toggled", self.cb_rg_indicate)
         vbox.pack_start(self.rg_indicate, False, False, 0)
         self.rg_indicate.show()
         
-        self.rg_adjust = gtk.CheckButton(_('Adjust playback volume'))
+        
+        
+        self.rg_adjust = gtk.CheckButton(_('Adjust playback volume in dB'))
         set_tip(self.rg_adjust, _('Effective only on newly started tracks.'))
         vbox.pack_start(self.rg_adjust, False, False, 0)
         self.rg_adjust.show()
         
-        hbox = gtk.HBox()
-        hbox.set_spacing(3)
-        spacer = gtk.HBox()
-        hbox.pack_start(spacer, False, False, 16)
-        spacer.show()
-        label = gtk.Label(_('Unmarked tracks assumed gain value'))
-        hbox.pack_start(label, False, False, 0)
-        label.show()
-        rg_defaultgainadj = gtk.Adjustment(-8.0, -20.0, 10.0, 0.1)
-        self.rg_defaultgain = gtk.SpinButton(rg_defaultgainadj, 0.0, 1)
-        set_tip(hbox, _('Set this to the typical track gain values you would '
-        'expect for the programme material you are currently playing. For pop '
-        'and rock music (especially modern studio recordings) this should be'
-        ' about a -8 or -9 and classical music around zero.'))
-        hbox.pack_start(self.rg_defaultgain, False, False, 0)
-        self.rg_defaultgain.show()
-        vbox.pack_start(hbox, False, False, 0)
-        hbox.show()
-
-        hbox = gtk.HBox()
-        hbox.set_spacing(3)
-        spacer = gtk.HBox()
-        hbox.pack_start(spacer, False, False, 16)
-        spacer.show()
-        label = gtk.Label(_('Further gain adjustment'))
-        hbox.pack_start(label, False, False, 0)
-        label.show()
-        rg_boostadj = gtk.Adjustment(6.0, -5.0, 15.5, 0.5)
+        table = gtk.Table(2, 6)
+        table.set_col_spacings(3)
+        label = gtk.Label(_('R128'))
+        label.set_alignment(1.0, 0.5)
+        r128_boostadj = gtk.Adjustment(4.0, -5.0, 25.5, 0.5)
+        self.r128_boost = gtk.SpinButton(r128_boostadj, 0.0, 1)
+        set_tip(self.r128_boost, _('It may not be desirable to use the '
+                    'default level since it is rather quiet. This should be'
+                    ' set 4 or 5 dB higher than the ReplayGain setting.'))
+        table.attach(label, 0, 1, 0, 1)
+        table.attach(self.r128_boost, 1, 2, 0, 1)
+        label = gtk.Label(_('ReplayGain'))
+        label.set_alignment(1.0, 0.5)
+        rg_boostadj = gtk.Adjustment(0.0, -10.0, 20.5, 0.5)
         self.rg_boost = gtk.SpinButton(rg_boostadj, 0.0, 1)
-        set_tip(hbox, _('For material that is generally loud it is recommended '
-        'to set this between 4 and 8 dB however going too high will result in a'
-        ' loss of dynamic range. The Str Peak meter is a useful guide for '
-        'getting this right.'))
-        hbox.pack_start(self.rg_boost, False, False, 0)
-        self.rg_boost.show()
-        vbox.pack_start(hbox, False, False, 0)
-        hbox.show()
+        set_tip(self.rg_boost, _('It may not be desirable to use the default'
+                        ' level since it is rather quiet. This should be set'
+                        ' 4 or 5 dB lower than the R128 setting.'))
+        table.attach(label, 2, 3, 0, 1)
+        table.attach(self.rg_boost, 3, 4, 0, 1)
+        label = gtk.Label(_('Untagged'))
+        label.set_alignment(1.0, 0.5)
+        rg_defaultgainadj = gtk.Adjustment(-8.0, -30.0, 10.0, 0.5)
+        self.rg_defaultgain = gtk.SpinButton(rg_defaultgainadj, 0.0, 1)
+        set_tip(self.rg_defaultgain, _('Set this so that any unmarked tracks'
+        ' are playing at a roughly similar loudness level as the marked ones.'))
+        table.attach(label, 4, 5, 0, 1)
+        table.attach(self.rg_defaultgain, 5, 6, 0, 1)
+
+        label = gtk.Label(_('All'))
+        label.set_alignment(1.0, 0.5)
+        all_boostadj = gtk.Adjustment(0.0, -10.0, 10.0, 0.5)
+        self.all_boost = gtk.SpinButton(all_boostadj, 0.0, 1)
+        set_tip(self.all_boost, _('A master level control for the media players.'))
+        table.attach(label, 0, 1, 1, 2)
+        table.attach(self.all_boost, 1, 2, 1, 2)
+        
+        vbox.pack_start(table, False)
+        table.set_col_spacing(1, 7)
+        table.set_col_spacing(3, 7)
+        table.show_all()
+
+        # Recorder filename format may be desirable to change for FAT32 compatibility
+
+        frame = gtk.Frame(" %s " % _('Recorder Filename (excluding the file extension)'))
+        set_tip(frame, _("The specifiers are $r for the number of the "
+        "recorder with the rest being documented in the strftime man page.\n"
+        "Users may wish to alter this to make filenames that are compatible with particular filesystems."))
+        frame.set_border_width(3)
+        align = gtk.Alignment()
+        align.props.xscale = 1.0
+        self.recorder_filename = DefaultEntry("idjc.[%Y-%m-%d][%H:%M:%S].$r")
+        align.add(self.recorder_filename)
+        self.recorder_filename.show()
+        align.set_border_width(3)
+        frame.add(align)
+        align.show()
+        outervbox.pack_start(frame, True)
+        frame.show()
 
         # Miscellaneous Features
         
@@ -1072,86 +1219,18 @@ class mixprefs:
         
         vbox.show()
 
-        outervbox.pack_start(frame, False, False, 0)
+        outervbox.pack_start(frame, False)
+       
+        # Song database preferences and connect button.
+        self.songdbprefs = self.parent.topleftpane.prefs_controls
+        self.parent.menu.songdbmenu_a.connect_proxy(self.songdbprefs.dbtoggle)
+        outervbox.pack_start(self.songdbprefs, False)
         
-        
-        aud_rs_hbox = gtk.HBox()
-        
-        # User can use this to set the audio level in the headphones
-        
-        # TC: The DJ's sound level controller.
-        frame = gtk.Frame(" " + _('DJ Audio Level') + " ")
-        frame.set_label_align(0.5, 0.5)
-        frame.set_border_width(3)
-        hbox = gtk.HBox()
-        hbox.set_border_width(5)
-        frame.add(hbox)
-        hbox.show()
-        
-        self.dj_aud_adj = gtk.Adjustment(0.0, -60.0, 0.0, 0.5, 1.0)
-        dj_aud = gtk.SpinButton(self.dj_aud_adj, 1, 1)
-        dj_aud.connect("value-changed", self.cb_dj_aud)
-        hbox.pack_start(dj_aud, True, False, 0)
-        dj_aud.show()
-        set_tip(dj_aud, _('This adjusts the sound level of the DJ audio.'))
-        
-        aud_rs_hbox.pack_start(frame, False, False, 0)
-        frame.show()
-        
-        # User can use this to set the resampled sound quality
-        
-        frame = gtk.Frame(" %s " % _('Player Resample Quality'))
-        frame.set_label_align(0.5, 0.5)
-        frame.set_border_width(3)
-        hbox = gtk.HBox()
-        hbox.set_border_width(5)
-        set_tip(hbox,
-        _('This adjusts the quality of the audio resampling method '
-        'used whenever the sample rate of the music file currently playing does'
-        ' not match the sample rate of the JACK sound server. Highest mode '
-        'offers the best sound quality but also uses the most CPU (not '
-        'recommended for systems built before 2006). All these modes provide '
-        'adequate sound quality.'))
-        frame.add(hbox)
-        hbox.show()
-        self.best_quality_resample = gtk.RadioButton(None, _('Highest'))
-        self.best_quality_resample.connect(
-                                        "toggled", self.cb_resample_quality, 0)
-        rsbox = gtk.HBox()
-        rsbox.pack_start(self.best_quality_resample, True, False, 0)
-        rsbox.show()
-        hbox.add(rsbox)
-        self.best_quality_resample.show()
-        self.good_quality_resample = gtk.RadioButton(
-                                        self.best_quality_resample, _('Medium'))
-        self.good_quality_resample.connect(
-                                        "toggled", self.cb_resample_quality, 1) 
-        rsbox = gtk.HBox()
-        rsbox.pack_start(self.good_quality_resample, True, False, 0)
-        rsbox.show()
-        hbox.add(rsbox)
-        self.good_quality_resample.show()
-        self.fast_resample = gtk.RadioButton(
-                                        self.good_quality_resample, _('Lowest'))
-        self.fast_resample.connect("toggled", self.cb_resample_quality, 2) 
-        rsbox = gtk.HBox()
-        rsbox.pack_start(self.fast_resample, True, False, 0)
-        rsbox.show()
-        hbox.add(rsbox)
-        self.fast_resample.show()
-
-        aud_rs_hbox.pack_start(frame, True, True, 0)
-        frame.show()
-        
-        outervbox.pack_start(aud_rs_hbox, False, False, 0)
-        aud_rs_hbox.show()
-        
-        # Prokyon 3 database connection
-        self.p3prefs = p3db.Prefs(self.parent)
-        outervbox.pack_start(self.p3prefs, False, False, 0)
-        self.p3prefs.show()
-        self.parent.menu.songdbmenu_a.connect_proxy(self.p3prefs.proktoggle)
-
+        # Widget for user interface label renaming.
+        label_subst = self.parent.label_subst
+        outervbox.pack_start(label_subst, False)
+        label_subst.set_border_width(3)
+        label_subst.show_all()
         
         # Session to be saved, or initial settings preferences.
         frame = gtk.Frame(" %s " % _('Player Settings At Startup'))
@@ -1217,7 +1296,7 @@ class mixprefs:
         self.restore_session_option.connect("toggled", self.cb_restore_session)
         self.restore_session_option.set_active(True)
       
-        outervbox.pack_start(frame, False, False, 0)
+        outervbox.pack_start(frame, False)
         frame.show() 
                 
         # TC: A heading label for miscellaneous settings.
@@ -1246,7 +1325,7 @@ class mixprefs:
 
         # Individual channel settings
 
-        mic_controls = []
+        self.mic_controls = mic_controls = []
         vbox = gtk.VBox()
         for i in range(PGlobs.num_micpairs):
             uhbox = gtk.HBox(True)
@@ -1259,6 +1338,7 @@ class mixprefs:
                 n = i * 2 + j
                 micname = "mic_control_%d" % n
                 c = AGCControl(self.parent, str(n + 1), micname, n)
+                setattr(self, micname, c)
                 uhbox.add(c)
                 c.show()
                 parent.mic_opener.add_mic(c)
@@ -1313,7 +1393,7 @@ class mixprefs:
         
         label = gtk.Label()
         label.set_markup(
-        '<span font_desc="sans 10" underline="low" foreground="blue">' +
+        '<span font_desc="sans 10">' +
          _('Released under the GNU General Public License V2.0') + '</span>')
         vbox.pack_start(label, False, False, 1)
         label.show()
@@ -1353,7 +1433,8 @@ class mixprefs:
             ("Stephen Fairchild (s-fairchild@users.sourceforge.net)",
             "And Clover (and@doxdesk.com)",
             "Dario Abatianni (eisfuchs@users.sourceforge.net)",
-            "Stefan Fendt (stefan@sfendt.de)"))
+            "Stefan Fendt (stefan@sfendt.de)",
+            "Brian Millham (bmillham@users.sourceforge.net)"))
 
         contribs_page(_('Translators'),
             ("<b>it</b>  Raffaele Morelli (raffaele.morelli@gmail.com)",))
@@ -1402,9 +1483,12 @@ class mixprefs:
 
         self.parent.menu.strmetersmenu_a.connect_proxy(self.show_stream_meters)
         self.parent.menu.chmetersmenu_a.connect_proxy(self.show_microphones)
+        self.parent.menu.backgroundtracksmenu_a.connect_proxy(self.show_background_tracks_player)
+        self.parent.menu.buttonbarmenu_a.connect_proxy(self.show_button_bar)
 
         self.show_stream_meters.set_active(True)
         self.show_microphones.set_active(True)
+        self.show_button_bar.set_active(True)
 
         self.activedict = {  # Widgets to save that have the get_active method.
             "startmini"   : self.startmini,
@@ -1415,7 +1499,6 @@ class mixprefs:
             "bigdigibox"  : self.bigger_box_toggle, 
             "dither"      : self.dither,
             "recallsession" : self.restore_session_option,
-            "proktoggle"    : self.p3prefs.proktoggle,
             "best_rs"       : self.best_quality_resample,
             "good_rs"       : self.good_quality_resample,
             "fast_rs"       : self.fast_resample,
@@ -1429,40 +1512,47 @@ class mixprefs:
             "str_meters"    : self.show_stream_meters,
             "mic_meters"    : self.show_microphones,
             "mic_meters_no_void" : self.no_mic_void_space,
-            "players_visible"    : self.parent.menu.playersmenu_i,
+            "players_visible"    : self.parent.menu.playersmenu_i
             }
             
         for each in itertools.chain(mic_controls, 
-                            (self.parent.freewheel_button,
-                            self.lpconfig, self.rpconfig, opener_settings)):
+                            (self.parent.freewheel_button, self.songdbprefs,
+                            self.lpconfig, self.rpconfig, opener_settings,
+                            label_subst)):
             self.activedict.update(each.activedict)
 
         self.valuesdict = {  # These widgets all have the get_value method.
-            "jingle_vol"    : self.parent.jingles.jvol_adj,
-            "jingle_muting" : self.parent.jingles.jmute_adj,
+            "effects1_vol"    : self.parent.jingles.jvol_adj[0],
+            "effects1_muting" : self.parent.jingles.jmute_adj[0],
+            "effects2_vol"    : self.parent.jingles.jvol_adj[1],
+            "effects2_muting" : self.parent.jingles.jmute_adj[1],
+            "voiplevel"     : self.parent.voipgainadj,
+            "voipmixback"   : self.parent.mixbackadj,
             "interlude_vol" : self.parent.jingles.ivol_adj,
             "passspeed"     : self.parent.passspeed_adj,
             "djvolume"      : self.dj_aud_adj,
+            "alarmvolume"   : self.alarm_aud_adj,
             "rg_default"    : self.rg_defaultgain,
             "rg_boost"      : self.rg_boost,
+            "r128_boost"    : self.r128_boost,
+            "all_boost"    : self.all_boost
             }
 
-        for each in itertools.chain(mic_controls, (opener_settings,)):
+        for each in itertools.chain(mic_controls, (opener_settings,
+                                                            self.songdbprefs)):
             self.valuesdict.update(each.valuesdict)
 
         self.textdict = {  # These widgets all have the get_text method.
-            "prokuser"      : self.p3prefs.prokuser,
-            "prokdatabase"  : self.p3prefs.prokdatabase,
-            "prokpassword"  : self.p3prefs.prokpassword,
-            "prokhostname"  : self.p3prefs.prokhostname,
             "ltfilerqdir"   : self.parent.player_left.file_requester_start_dir,
             "rtfilerqdir"   : self.parent.player_right.file_requester_start_dir,
             "main_full_wst" : self.parent.full_wst,
             "main_min_wst"  : self.parent.min_wst,
-            "prefs_wst"   : self.wst,
+            "prefs_wst"     : self.wst,
+            "rec_filename"  : self.recorder_filename
             }
 
-        for each in itertools.chain(mic_controls, (opener_settings,)):
+        for each in itertools.chain(mic_controls, (opener_settings,
+                                            label_subst, self.songdbprefs)):
             self.textdict.update(each.textdict)
 
         self.rangewidgets = (self.parent.deckadj,)
